# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

"""This module contains some helpers for mocking eks clusters"""

import os

EXAMPLE_NAME = "ExampleCluster"


def get_testdata(file_name):
    """Get the path of a specific fixture"""
    return os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "testdata", file_name
    )


def list_cluster_response():
    """Get an example list_cluster call (For mocking)"""
    return {"clusters": [EXAMPLE_NAME]}


def describe_cluster_response():
    """Get an example describe_cluster call (For mocking)"""
    return {
        "cluster": {
            "status": "ACTIVE",
            "endpoint": "https://endpoint.amazonaws.com",
            "name": EXAMPLE_NAME,
            "certificateAuthority": {
                "data": "LS0tLS1CRUdJTiBDRVJUSUZJQ0FURS0tLS0tDQpWR1Z6ZEdsdVp5QkVZWFJoRFFwVVpYTjBhVzVuSUVSaGRHRU5DbFJsYzNScGJtY2dSR0YwWVEwS2EzVmlaWEp1WlhSbGN6QWVGdzBLVkdWemRHbHVaeUJFWVhSaERRcFVaWE4wYVc1bklFUmhkR0ZWQkFNVERRcHJkV0psY201bGRHVnpNQUVpTUEwS1ZHVnpkR2x1WnlCRVlYUmhEUXBVWlhOMGFXNW5JRVJoZEdFTkNsUmxjM1JwYm1jZ1JHRjBZY3UvR1FnbmFTcDNZaHBDTWhGVVpYTjBhVzVuSUVSaGRHRXl3clZqeEpWNjNwNFVHRmpZdHdGR1drUldJVkV1VkdWemRHbHVaeUJFWVhSaGJzT0MxSVJiTDhPd0lpMVhiWGg2VkdWemRHbHVaeUJFWVhSaFpXVndTTk9VVUZKNmN5QWJaaFpnWVNkTUV3MEtGMVJsYzNScGJtY2dSR0YwWVFZRFZSMFBBUUVFQkFNQ0FsUmxjM1JwYm1jZ1JHRjBZUUV3RFFvR0NTcElEUXBVWlhOMGFXNW5JRVJoZEdGcEgxc1pPRTNMa3lrMU9DWUNHUloyTEZjM3paOCtHell3WEZSbGMzUnBibWNnUkdGMFlYMUR5NjFNMVlGV1AxWVRIMVJsYzNScGJtY2dSR0YwWVd0aE5oMVphM2dWUDBGaGNSWjdKaW9oZVc4N1JsUmxjM1JwYm1jZ1JHRjBZUVpIVHd4NE9IdzZmZz09DQotLS0tLUVORCBDRVJUSUZJQ0FURS0tLS0t"
            },
            "roleArn": "arn:aws:iam::111222333444/eksRole",
            "resourcesVpcConfig": {
                "subnetIds": [
                    "subnet-00000000000000000",
                    "subnet-00000000000000001",
                    "subnet-00000000000000002",
                ],
                "vpcId": "vpc-00000000000000000",
                "securityGroupIds": ["sg-00000000000000000"],
            },
            "version": "1.10",
            "arn": "arn:aws:eks:region:111222333444:cluster/" + EXAMPLE_NAME,
            "createdAt": 1500000000.000,
        }
    }


def describe_cluster_response_outpost_cluster():
    """Get an example describe_cluster call (For mocking)"""
    return {
        "cluster": {
            "status": "ACTIVE",
            "endpoint": "https://endpoint.amazonaws.com",
            "name": EXAMPLE_NAME,
            "certificateAuthority": {
                "data": "LS0tLS1CRUdJTiBDRVJUSUZJQ0FURS0tLS0tDQpWR1Z6ZEdsdVp5QkVZWFJoRFFwVVpYTjBhVzVuSUVSaGRHRU5DbFJsYzNScGJtY2dSR0YwWVEwS2EzVmlaWEp1WlhSbGN6QWVGdzBLVkdWemRHbHVaeUJFWVhSaERRcFVaWE4wYVc1bklFUmhkR0ZWQkFNVERRcHJkV0psY201bGRHVnpNQUVpTUEwS1ZHVnpkR2x1WnlCRVlYUmhEUXBVWlhOMGFXNW5JRVJoZEdFTkNsUmxjM1JwYm1jZ1JHRjBZY3UvR1FnbmFTcDNZaHBDTWhGVVpYTjBhVzVuSUVSaGRHRXl3clZqeEpWNjNwNFVHRmpZdHdGR1drUldJVkV1VkdWemRHbHVaeUJFWVhSaGJzT0MxSVJiTDhPd0lpMVhiWGg2VkdWemRHbHVaeUJFWVhSaFpXVndTTk9VVUZKNmN5QWJaaFpnWVNkTUV3MEtGMVJsYzNScGJtY2dSR0YwWVFZRFZSMFBBUUVFQkFNQ0FsUmxjM1JwYm1jZ1JHRjBZUUV3RFFvR0NTcElEUXBVWlhOMGFXNW5JRVJoZEdGcEgxc1pPRTNMa3lrMU9DWUNHUloyTEZjM3paOCtHell3WEZSbGMzUnBibWNnUkdGMFlYMUR5NjFNMVlGV1AxWVRIMVJsYzNScGJtY2dSR0YwWVd0aE5oMVphM2dWUDBGaGNSWjdKaW9oZVc4N1JsUmxjM1JwYm1jZ1JHRjBZUVpIVHd4NE9IdzZmZz09DQotLS0tLUVORCBDRVJUSUZJQ0FURS0tLS0t"
            },
            "roleArn": "arn:aws:iam::111222333444/eksRole",
            "resourcesVpcConfig": {
                "subnetIds": [
                    "subnet-00000000000000000",
                    "subnet-00000000000000001",
                    "subnet-00000000000000002",
                ],
                "vpcId": "vpc-00000000000000000",
                "securityGroupIds": ["sg-00000000000000000"],
            },
            "version": "1.10",
            "arn": "arn:aws:eks:region:111222333444:cluster/" + EXAMPLE_NAME,
            "createdAt": 1500000000.000,
            "id": "aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee",
            "outpostConfig": {
                "outpostArns": [
                    "arn:aws:outposts:us-west-2:111222333444:outpost/op-00000000000000000"
                ],
            },
        }
    }


def describe_cluster_no_status_response():
    """Get an example describe_cluster call (For mocking)"""
    return {
        "cluster": {
            "endpoint": "https://endpoint.amazonaws.com",
            "name": EXAMPLE_NAME,
            "certificateAuthority": {
                "data": "LS0tLS1CRUdJTiBDRVJUSUZJQ0FURS0tLS0tDQpWR1Z6ZEdsdVp5QkVZWFJoRFFwVVpYTjBhVzVuSUVSaGRHRU5DbFJsYzNScGJtY2dSR0YwWVEwS2EzVmlaWEp1WlhSbGN6QWVGdzBLVkdWemRHbHVaeUJFWVhSaERRcFVaWE4wYVc1bklFUmhkR0ZWQkFNVERRcHJkV0psY201bGRHVnpNQUVpTUEwS1ZHVnpkR2x1WnlCRVlYUmhEUXBVWlhOMGFXNW5JRVJoZEdFTkNsUmxjM1JwYm1jZ1JHRjBZY3UvR1FnbmFTcDNZaHBDTWhGVVpYTjBhVzVuSUVSaGRHRXl3clZqeEpWNjNwNFVHRmpZdHdGR1drUldJVkV1VkdWemRHbHVaeUJFWVhSaGJzT0MxSVJiTDhPd0lpMVhiWGg2VkdWemRHbHVaeUJFWVhSaFpXVndTTk9VVUZKNmN5QWJaaFpnWVNkTUV3MEtGMVJsYzNScGJtY2dSR0YwWVFZRFZSMFBBUUVFQkFNQ0FsUmxjM1JwYm1jZ1JHRjBZUUV3RFFvR0NTcElEUXBVWlhOMGFXNW5JRVJoZEdGcEgxc1pPRTNMa3lrMU9DWUNHUloyTEZjM3paOCtHell3WEZSbGMzUnBibWNnUkdGMFlYMUR5NjFNMVlGV1AxWVRIMVJsYzNScGJtY2dSR0YwWVd0aE5oMVphM2dWUDBGaGNSWjdKaW9oZVc4N1JsUmxjM1JwYm1jZ1JHRjBZUVpIVHd4NE9IdzZmZz09DQotLS0tLUVORCBDRVJUSUZJQ0FURS0tLS0t"
            },
            "roleArn": "arn:aws:iam::111222333444/eksRole",
            "resourcesVpcConfig": {
                "subnetIds": [
                    "subnet-00000000000000000",
                    "subnet-00000000000000001",
                    "subnet-00000000000000002",
                ],
                "vpcId": "vpc-00000000000000000",
                "securityGroupIds": ["sg-00000000000000000"],
            },
            "version": "1.10",
            "arn": "arn:aws:eks:region:111222333444:cluster/" + EXAMPLE_NAME,
            "createdAt": 1500000000.000,
        }
    }


def describe_cluster_creating_response():
    """Get an example describe_cluster call during creation"""
    return {
        "cluster": {
            "status": "CREATING",
            "name": EXAMPLE_NAME,
            "certificateAuthority": {},
            "roleArn": "arn:aws:iam::111222333444/eksRole",
            "resourcesVpcConfig": {
                "subnetIds": [
                    "subnet-00000000000000000",
                    "subnet-00000000000000001",
                    "subnet-00000000000000002",
                ],
                "vpcId": "vpc-00000000000000000",
                "securityGroupIds": ["sg-00000000000000000"],
            },
            "version": "1.10",
            "arn": "arn:aws:eks:region:111222333444:cluster/" + EXAMPLE_NAME,
            "createdAt": 1500000000.000,
        }
    }


def describe_cluster_deleting_response():
    """Get an example describe_cluster call during deletion"""
    return {
        "cluster": {
            "status": "DELETING",
            "endpoint": "https://endpoint.amazonaws.com",
            "name": EXAMPLE_NAME,
            "certificateAuthority": {
                "data": "LS0tLS1CRUdJTiBDRVJUSUZJQ0FURS0tLS0tDQpWR1Z6ZEdsdVp5QkVZWFJoRFFwVVpYTjBhVzVuSUVSaGRHRU5DbFJsYzNScGJtY2dSR0YwWVEwS2EzVmlaWEp1WlhSbGN6QWVGdzBLVkdWemRHbHVaeUJFWVhSaERRcFVaWE4wYVc1bklFUmhkR0ZWQkFNVERRcHJkV0psY201bGRHVnpNQUVpTUEwS1ZHVnpkR2x1WnlCRVlYUmhEUXBVWlhOMGFXNW5JRVJoZEdFTkNsUmxjM1JwYm1jZ1JHRjBZY3UvR1FnbmFTcDNZaHBDTWhGVVpYTjBhVzVuSUVSaGRHRXl3clZqeEpWNjNwNFVHRmpZdHdGR1drUldJVkV1VkdWemRHbHVaeUJFWVhSaGJzT0MxSVJiTDhPd0lpMVhiWGg2VkdWemRHbHVaeUJFWVhSaFpXVndTTk9VVUZKNmN5QWJaaFpnWVNkTUV3MEtGMVJsYzNScGJtY2dSR0YwWVFZRFZSMFBBUUVFQkFNQ0FsUmxjM1JwYm1jZ1JHRjBZUUV3RFFvR0NTcElEUXBVWlhOMGFXNW5JRVJoZEdGcEgxc1pPRTNMa3lrMU9DWUNHUloyTEZjM3paOCtHell3WEZSbGMzUnBibWNnUkdGMFlYMUR5NjFNMVlGV1AxWVRIMVJsYzNScGJtY2dSR0YwWVd0aE5oMVphM2dWUDBGaGNSWjdKaW9oZVc4N1JsUmxjM1JwYm1jZ1JHRjBZUVpIVHd4NE9IdzZmZz09DQotLS0tLUVORCBDRVJUSUZJQ0FURS0tLS0t"
            },
            "roleArn": "arn:aws:iam::111222333444/eksRole",
            "resourcesVpcConfig": {
                "subnetIds": [
                    "subnet-00000000000000000",
                    "subnet-00000000000000001",
                    "subnet-00000000000000002",
                ],
                "vpcId": "vpc-00000000000000000",
                "securityGroupIds": ["sg-00000000000000000"],
            },
            "version": "1.10",
            "arn": "arn:aws:eks:region:111222333444:cluster/" + EXAMPLE_NAME,
            "createdAt": 1500000000.000,
        }
    }


def assume_role_response():
    return {
        "Credentials": {
            "AccessKeyId": "test-access-key",
            "SecretAccessKey": "test-secret-key",
            "SessionToken": "test-session-token",
        }
    }



# Copyright 2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import json

from awscli.testutils import BaseAWSCommandParamsTest


class TestPagination(BaseAWSCommandParamsTest):
    def setUp(self):
        super(TestPagination, self).setUp()
        self.first_response = {
            "Items": [{"Key": {"B": "MjEzNw=="}}],
            "Count": 1,
            "ScannedCount": 1,
            "ConsumedCapacity": 1,
            "LastEvaluatedKey": {"Key": {"B": "MjEzNw=="}},
        }
        self.second_response = {
            "Items": [],
            "Count": 0,
            "ScannedCount": 1,
            "ConsumedCapacity": 1,
        }

    def test_scan_pagination_binary_last_evaluated_key(self):
        self.parsed_responses = [self.first_response, self.second_response]
        self.run_cmd('dynamodb scan --table-name test', expected_rc=0)
        self.assertEqual(len(self.operations_called), 2)
        # The start key in the second request should have been parsed to binary
        sent_start_key = self.operations_called[1][1].get('ExclusiveStartKey')
        expected_start_key = {"Key": {"B": b'2137'}}
        self.assertEqual(sent_start_key, expected_start_key)

    def test_pagination_disabled_works(self):
        self.parsed_responses = [self.first_response]
        cmd = 'dynamodb scan --table-name table --no-paginate --output json'
        stdout, _, _ = self.run_cmd(cmd, expected_rc=0)
        # Ensure the base64 encoded last evaluated key is in stdout
        self.assertIn('"MjEzNw=="', stdout)



# Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import argparse
import os

from botocore.exceptions import ProfileNotFound
from botocore.history import HistoryRecorder
from botocore.session import Session

from awscli.compat import StringIO
from awscli.customizations.exceptions import ParamValidationError
from awscli.customizations.history import (
    HistoryCommand,
    add_history_commands,
    attach_history_handler,
)
from awscli.customizations.history.db import DatabaseHistoryHandler
from awscli.testutils import FileCreator, mock, unittest


class TestAttachHistoryHandler(unittest.TestCase):
    def setUp(self):
        self.files = FileCreator()

    def tearDown(self):
        self.files.remove_all()

    @mock.patch('awscli.customizations.history.sqlite3')
    @mock.patch('awscli.customizations.history.db.sqlite3')
    @mock.patch(
        'awscli.customizations.history.HISTORY_RECORDER', spec=HistoryRecorder
    )
    def test_attach_history_handler(
        self, mock_recorder, mock_db_sqlite3, mock_sqlite3
    ):
        mock_session = mock.Mock(Session)
        mock_session.get_scoped_config.return_value = {
            'cli_history': 'enabled'
        }

        parsed_args = argparse.Namespace()
        parsed_args.command = 's3'

        attach_history_handler(session=mock_session, parsed_args=parsed_args)
        self.assertEqual(mock_recorder.add_handler.call_count, 1)
        self.assertIsInstance(
            mock_recorder.add_handler.call_args[0][0], DatabaseHistoryHandler
        )
        self.assertTrue(mock_db_sqlite3.connect.called)

    @mock.patch('awscli.customizations.history.sqlite3')
    @mock.patch('awscli.customizations.history.db.sqlite3')
    @mock.patch(
        'awscli.customizations.history.HISTORY_RECORDER', spec=HistoryRecorder
    )
    def test_no_attach_history_handler_when_history_not_configured(
        self, mock_recorder, mock_db_sqlite3, mock_sqlite3
    ):
        mock_session = mock.Mock(Session)
        mock_session.get_scoped_config.return_value = {}

        parsed_args = argparse.Namespace()
        parsed_args.command = 's3'

        attach_history_handler(session=mock_session, parsed_args=parsed_args)
        self.assertFalse(mock_recorder.add_handler.called)
        self.assertFalse(mock_db_sqlite3.connect.called)

    @mock.patch('awscli.customizations.history.sqlite3')
    @mock.patch('awscli.customizations.history.db.sqlite3')
    @mock.patch(
        'awscli.customizations.history.HISTORY_RECORDER', spec=HistoryRecorder
    )
    def test_no_attach_history_handler_when_command_is_history(
        self, mock_recorder, mock_db_sqlite3, mock_sqlite3
    ):
        mock_session = mock.Mock(Session)
        mock_session.get_scoped_config.return_value = {
            'cli_history': 'enabled'
        }

        parsed_args = argparse.Namespace()
        parsed_args.command = 'history'

        attach_history_handler(session=mock_session, parsed_args=parsed_args)
        self.assertFalse(mock_recorder.add_handler.called)
        self.assertFalse(mock_db_sqlite3.connect.called)

    @mock.patch('awscli.customizations.history.sqlite3', None)
    @mock.patch('awscli.customizations.history.db.sqlite3')
    @mock.patch(
        'awscli.customizations.history.HISTORY_RECORDER', spec=HistoryRecorder
    )
    def test_no_attach_history_handler_when_no_sqlite3(
        self, mock_recorder, mock_sqlite3
    ):
        mock_session = mock.Mock(Session)
        mock_session.get_scoped_config.return_value = {
            'cli_history': 'enabled'
        }

        parsed_args = argparse.Namespace()
        parsed_args.command = 's3'

        with mock.patch('sys.stderr', StringIO()) as mock_stderr:
            attach_history_handler(
                session=mock_session, parsed_args=parsed_args
            )
            self.assertIn(
                'enabled but sqlite3 is unavailable', mock_stderr.getvalue()
            )
        self.assertFalse(mock_recorder.add_handler.called)
        self.assertFalse(mock_sqlite3.connect.called)

    @mock.patch('awscli.customizations.history.sqlite3')
    @mock.patch('awscli.customizations.history.db.sqlite3')
    @mock.patch(
        'awscli.customizations.history.HISTORY_RECORDER', spec=HistoryRecorder
    )
    def test_create_directory_no_exists(
        self, mock_recorder, mock_db_sqlite3, mock_sqlite3
    ):
        mock_session = mock.Mock(Session)
        mock_session.get_scoped_config.return_value = {
            'cli_history': 'enabled'
        }

        parsed_args = argparse.Namespace()
        parsed_args.command = 's3'

        directory_to_create = os.path.join(self.files.rootdir, 'create-dir')
        db_filename = os.path.join(directory_to_create, 'name.db')
        with mock.patch('os.environ', {'AWS_CLI_HISTORY_FILE': db_filename}):
            attach_history_handler(
                session=mock_session, parsed_args=parsed_args
            )
            self.assertEqual(mock_recorder.add_handler.call_count, 1)
            # Is should create any missing parent directories of the
            # file as well.
            self.assertTrue(os.path.exists(directory_to_create))
        self.assertTrue(mock_db_sqlite3.connect.called)

    @mock.patch('awscli.customizations.history.sqlite3')
    @mock.patch('awscli.customizations.history.db.sqlite3')
    @mock.patch(
        'awscli.customizations.history.HISTORY_RECORDER', spec=HistoryRecorder
    )
    def test_profile_does_not_exist(
        self, mock_recorder, mock_db_sqlite3, mock_sqlite3
    ):
        mock_session = mock.Mock(Session)
        mock_session.get_scoped_config.side_effect = ProfileNotFound(
            profile='no-exist'
        )

        parsed_args = argparse.Namespace()
        parsed_args.command = 'configure'

        attach_history_handler(session=mock_session, parsed_args=parsed_args)
        self.assertFalse(mock_recorder.add_handler.called)
        self.assertFalse(mock_db_sqlite3.connect.called)

    @mock.patch('awscli.customizations.history.DatabaseConnection')
    @mock.patch(
        'awscli.customizations.history.HISTORY_RECORDER', spec=HistoryRecorder
    )
    def test_warning_when_database_connection_fails(
        self, mock_recorder, mock_db_connection
    ):
        mock_db_connection.side_effect = Exception('Permission denied')
        mock_session = mock.Mock(Session)
        mock_session.get_scoped_config.return_value = {
            'cli_history': 'enabled'
        }

        parsed_args = argparse.Namespace()
        parsed_args.command = 's3'

        with mock.patch('sys.stderr', new_callable=StringIO) as mock_stderr:
            attach_history_handler(
                session=mock_session, parsed_args=parsed_args
            )
            self.assertIn('Warning: Unable to record CLI history', mock_stderr.getvalue())
        self.assertFalse(mock_recorder.add_handler.called)


class TestAddHistoryCommand(unittest.TestCase):
    def test_add_history_command(self):
        command_table = {}
        mock_session = mock.Mock(Session)
        add_history_commands(command_table=command_table, session=mock_session)
        self.assertIn('history', command_table)
        self.assertIsInstance(command_table['history'], HistoryCommand)


class TestHistoryCommand(unittest.TestCase):
    def test_requires_subcommand(self):
        mock_session = mock.Mock(Session)
        history_command = HistoryCommand(mock_session)
        parsed_args = argparse.Namespace()
        parsed_args.subcommand = None
        parsed_globals = argparse.Namespace()
        with self.assertRaises(ParamValidationError):
            history_command._run_main(parsed_args, parsed_globals)



# Copyright 2022 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import os

import pytest
from build_system.constants import (
    BIN_DIRNAME,
    EXE_ASSETS_DIR,
    PYINSTALLER_DIR,
    PYINSTALLER_EXE_NAME,
)
from build_system.exe import ExeBuilder


class FakeUtils:
    def __init__(self, prior_build_dir=False):
        self._prior_build_dir = prior_build_dir
        self.calls = []

    def isdir(self, path: str) -> bool:
        self.calls.append(("isdir", path))
        return self._prior_build_dir

    def run(self, command, **kwargs):
        self.calls.append(("run", command, kwargs))

    def copy_directory(self, src, dst):
        self.calls.append(("copy_directory", src, dst))

    def copy_directory_contents_into(self, src, dst):
        self.calls.append(("copy_directory_contents_into", src, dst))

    def update_metadata(self, dirname, **kwargs):
        self.calls.append(("update_metadata", dirname, kwargs))

    def rmtree(self, path):
        self.calls.append(("rmtree", path))

    def glob(self, pattern, root, recursive=True):
        self.calls.append(('glob', pattern, root, recursive))
        return []


class FakeAwsCliVenv:
    @property
    def bin_dir(self) -> str:
        return BIN_DIRNAME

    @property
    def python_exe(self) -> str:
        return "python"


@pytest.fixture
def fake_aws_cli_venv():
    return FakeAwsCliVenv()


class TestExe:
    def _expected_build_tasks(self):
        return [
            # Check if build dir is present
            ("isdir", os.path.join("workspace", "dist")),
            # Build aws and copy into final directory
            (
                "run",
                [
                    "python",
                    os.path.join(BIN_DIRNAME, PYINSTALLER_EXE_NAME),
                    os.path.join(PYINSTALLER_DIR, "aws.spec"),
                    "--distpath",
                    os.path.join("workspace", "dist"),
                    "--workpath",
                    os.path.join("workspace", "build"),
                ],
                {"cwd": PYINSTALLER_DIR, "check": True},
            ),
            (
                "copy_directory",
                os.path.join("workspace", "dist", "aws"),
                os.path.join("workspace", "aws", "dist"),
            ),
            # build aws_completer and copy into final directory
            (
                "run",
                [
                    "python",
                    os.path.join(BIN_DIRNAME, PYINSTALLER_EXE_NAME),
                    os.path.join(PYINSTALLER_DIR, "aws_completer.spec"),
                    "--distpath",
                    os.path.join("workspace", "dist"),
                    "--workpath",
                    os.path.join("workspace", "build"),
                ],
                {"cwd": PYINSTALLER_DIR, "check": True},
            ),
            (
                "copy_directory_contents_into",
                os.path.join("workspace", "dist", "aws_completer"),
                os.path.join("workspace", "aws", "dist"),
            ),
            # Copy exe assets
            (
                "copy_directory_contents_into",
                EXE_ASSETS_DIR,
                os.path.join("workspace", "aws"),
            ),
            # Update metadata
            (
                "update_metadata",
                os.path.join("workspace", "aws", "dist"),
                {"distribution_source": "source-exe"},
            ),
            (
                "glob",
                "**/*.dist-info",
                os.path.join("workspace", "aws", "dist"),
                True,
            ),
        ]

    def test_build(self, fake_aws_cli_venv):
        fake_utils = FakeUtils()
        builder = ExeBuilder("workspace", fake_aws_cli_venv, _utils=fake_utils)
        builder.build()

        assert fake_utils.calls == self._expected_build_tasks() + [
            # Cleanup
            ("rmtree", os.path.join("workspace", "build")),
            ("rmtree", os.path.join("workspace", "dist")),
        ]

    def test_build_no_cleanup(self, fake_aws_cli_venv):
        fake_utils = FakeUtils()
        builder = ExeBuilder("workspace", fake_aws_cli_venv, _utils=fake_utils)
        builder.build(cleanup=False)

        assert fake_utils.calls == self._expected_build_tasks()

    def test_build_does_delete_prior_workspace(self, fake_aws_cli_venv):
        fake_utils = FakeUtils(prior_build_dir=True)

        builder = ExeBuilder("workspace", fake_aws_cli_venv, _utils=fake_utils)
        builder.build()

        assert fake_utils.calls[0:2] == [
            ("isdir", os.path.join("workspace", "dist")),
            ("rmtree", os.path.join("workspace", "dist")),
        ]



# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import os
from datetime import datetime

from botocore.awsrequest import AWSRequest
from botocore.credentials import Credentials
from botocore.session import Session

import awscli
from awscli.clidriver import AWSCLIEntryPoint, create_clidriver
from awscli.compat import StringIO
from awscli.customizations.codecommit import CodeCommitGetCommand
from awscli.testutils import StringIOWithFileNo, mock, unittest


class TestCodeCommitCredentialHelper(unittest.TestCase):
    PROTOCOL_HOST_PATH = (
        'protocol=https\n'
        'host=git-codecommit.us-east-1.amazonaws.com\n'
        'path=/v1/repos/myrepo'
    )

    FIPS_PROTOCOL_HOST_PATH = (
        'protocol=https\n'
        'host=git-codecommit-fips.us-east-1.amazonaws.com\n'
        'path=/v1/repos/myrepo'
    )

    VPC_PROTOCOL_HOST_PATH = (
        'protocol=https\n'
        'host=vpce-0b47ea360adebf88a-jkl88hez.git-codecommit.us-east-1.vpce.amazonaws.com\n'
        'path=/v1/repos/myrepo'
    )

    def setUp(self):
        self.orig_id = os.environ.get('AWS_ACCESS_KEY_ID')
        self.orig_key = os.environ.get('AWS_SECRET_ACCESS_KEY')
        os.environ['AWS_ACCESS_KEY_ID'] = 'foo'
        os.environ['AWS_SECRET_ACCESS_KEY'] = 'bar'

    def tearDown(self):
        if self.orig_id:
            os.environ['AWS_ACCESS_KEY_ID'] = self.orig_id
        else:
            del os.environ['AWS_ACCESS_KEY_ID']
        if self.orig_key:
            os.environ['AWS_SECRET_ACCESS_KEY'] = self.orig_key
        else:
            del os.environ['AWS_SECRET_ACCESS_KEY']

    @mock.patch('sys.stdin', StringIO(PROTOCOL_HOST_PATH))
    @mock.patch('sys.stdout', new_callable=StringIOWithFileNo)
    @mock.patch.object(awscli.customizations.codecommit.datetime, 'datetime')
    def test_integration_using_cli_driver(self, dt_mock, stdout_mock):
        dt_mock.utcnow.return_value = datetime(2010, 10, 8)
        driver = create_clidriver()
        entry_point = AWSCLIEntryPoint(driver)
        rc = entry_point.main('codecommit credential-helper get'.split())
        output = stdout_mock.getvalue().strip()
        self.assertEqual(
            (
                'username=foo\n'
                'password=20101008T000000Z'
                '7dc259e2d505af354a1219b9bcd784bd384dc706efa0d9aefc571f214be4c89c'
            ),
            output,
        )
        self.assertEqual(0, rc)

    @mock.patch('sys.stdin', StringIO(FIPS_PROTOCOL_HOST_PATH))
    @mock.patch('sys.stdout', new_callable=StringIOWithFileNo)
    @mock.patch.object(awscli.customizations.codecommit.datetime, 'datetime')
    def test_integration_fips_using_cli_driver(self, dt_mock, stdout_mock):
        dt_mock.utcnow.return_value = datetime(2010, 10, 8)
        driver = create_clidriver()
        entry_point = AWSCLIEntryPoint(driver)
        rc = entry_point.main('codecommit credential-helper get'.split())
        output = stdout_mock.getvalue().strip()
        self.assertEqual(
            (
                'username=foo\n'
                'password=20101008T000000Z'
                '500037cb3514b3fe01ebcda7c80973f5b4c0d8199a7a6563b85fd6edf272d460'
            ),
            output,
        )
        self.assertEqual(0, rc)

    @mock.patch('sys.stdin', StringIO(VPC_PROTOCOL_HOST_PATH))
    @mock.patch('sys.stdout', new_callable=StringIOWithFileNo)
    @mock.patch.object(awscli.customizations.codecommit.datetime, 'datetime')
    def test_integration_vpc_using_cli_driver(self, dt_mock, stdout_mock):
        dt_mock.utcnow.return_value = datetime(2010, 10, 8)
        driver = create_clidriver()
        entry_point = AWSCLIEntryPoint(driver)
        rc = entry_point.main('codecommit credential-helper get'.split())
        output = stdout_mock.getvalue().strip()
        self.assertEqual(
            (
                'username=foo\n'
                'password=20101008T000000Z'
                '9ed987cc6336c3de2d9f06b9236c7a9fd76b660b080db15983290e636dbfbd6b'
            ),
            output,
        )
        self.assertEqual(0, rc)


if __name__ == "__main__":
    unittest.main()



# Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from botocore import model
from botocore.exceptions import ProfileNotFound
from botocore.session import Session

from awscli.customizations.binaryformat import (
    Base64DecodeVisitor,
    InvalidBase64Error,
    add_binary_formatter,
    base64_decode_input_blobs,
    identity,
)
from awscli.testutils import mock, unittest


def test_identity_function():
    assert 2 == identity(2)


class TestBase64DecodeVisitor(unittest.TestCase):
    def construct_model(self, members):
        model_builder = model.DenormalizedStructureBuilder()
        return model_builder.with_members(members).build_model()

    def assert_decoded_params(self, members, params, expected_params):
        shape = self.construct_model(members)
        base64_decode_visitor = Base64DecodeVisitor()
        base64_decode_visitor.visit(params, shape)
        self.assertEqual(params, expected_params)

    def test_can_convert_top_level_blob(self):
        members = {'B': {'type': 'blob'}}
        params = {'B': 'Zm9vAGJheg=='}
        expected_params = {'B': b'foo\x00baz'}
        self.assert_decoded_params(members, params, expected_params)

    def test_can_convert_nested_blob(self):
        members = {
            'Nested': {
                'type': 'structure',
                'members': {
                    'B': {'type': 'blob'},
                },
            }
        }
        params = {'Nested': {'B': 'Zm9v'}}
        expected_params = {'Nested': {'B': b'foo'}}
        self.assert_decoded_params(members, params, expected_params)

    def test_can_convert_list_of_blob(self):
        members = {
            'BS': {
                'type': 'list',
                'member': {'type': 'blob'},
            }
        }
        params = {'BS': ['Zm9v', '']}
        expected_params = {'BS': [b'foo', b'']}
        self.assert_decoded_params(members, params, expected_params)

    def test_can_convert_map_to_blob(self):
        members = {
            'StoB': {
                'type': 'map',
                'key': {'type': 'string'},
                'value': {'type': 'blob'},
            }
        }
        params = {'StoB': {'a': 'Zm9v', 'b': ''}}
        expected_params = {'StoB': {'a': b'foo', 'b': b''}}
        self.assert_decoded_params(members, params, expected_params)

    def test_converts_base64_exception(self):
        members = {'B': {'type': 'blob'}}
        params = {'B': 'this:is:not:base64'}
        shape = self.construct_model(members)
        base64_decode_visitor = Base64DecodeVisitor()
        with self.assertRaises(InvalidBase64Error):
            base64_decode_visitor.visit(params, shape)


class TestAddBinaryFormatter(unittest.TestCase):
    def setUp(self):
        self.parsed_args = mock.Mock()
        self.mock_factory = mock.Mock()
        self.mock_session = mock.Mock(spec=Session)
        self.mock_session.get_component.return_value = self.mock_factory

    def _assert_legacy_handlers_added(self):
        # Legacy format does not register a handler to transform input
        self.assertEqual(self.mock_session.register.call_count, 0)
        # Legacy format parses blobs with an identity function
        self.mock_factory.set_parser_defaults.assert_called_with(
            blob_parser=identity
        )

    def test_legacy_handlers_added(self):
        self.parsed_args.cli_binary_format = 'raw-in-base64-out'
        add_binary_formatter(self.mock_session, self.parsed_args)
        self._assert_legacy_handlers_added()

    def test_legacy_handlers_added_via_profile(self):
        self.parsed_args.cli_binary_format = None
        self.mock_session.get_config_variable.return_value = (
            'raw-in-base64-out'
        )
        add_binary_formatter(self.mock_session, self.parsed_args)
        self._assert_legacy_handlers_added()

    def _assert_base64_handlers_added(self):
        self.assertEqual(self.mock_session.register.call_count, 1)
        self.mock_session.register.assert_called_with(
            'provide-client-params',
            base64_decode_input_blobs,
        )
        # base64 format parses blobs with an identity function
        self.mock_factory.set_parser_defaults.assert_called_with(
            blob_parser=identity
        )

    def test_base64_handlers_added(self):
        self.parsed_args.cli_binary_format = 'base64'
        add_binary_formatter(self.mock_session, self.parsed_args)
        self._assert_base64_handlers_added()

    def test_base64_handlers_added_via_profile(self):
        self.parsed_args.cli_binary_format = None
        self.mock_session.get_config_variable.return_value = 'base64'
        add_binary_formatter(self.mock_session, self.parsed_args)
        self._assert_base64_handlers_added()

    def test_defaults_to_base64_if_unkonwn_profile(self):
        self.parsed_args.cli_binary_format = None
        exception = ProfileNotFound(profile='notaprofile')
        self.mock_session.get_config_variable.side_effect = exception
        add_binary_formatter(self.mock_session, self.parsed_args)
        self._assert_base64_handlers_added()



# Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import json
import re
import threading

from awscli.compat import queue, sqlite3
from awscli.customizations.history.db import (
    DatabaseConnection,
    DatabaseHistoryHandler,
    DatabaseRecordReader,
    DatabaseRecordWriter,
    RecordBuilder,
)
from awscli.testutils import unittest
from tests import CaseInsensitiveDict


class ThreadedRecordWriter:
    def __init__(self, writer):
        self._read_q = queue.Queue()
        self._thread = threading.Thread(
            target=self._threaded_record_writer, args=(writer,)
        )

    def _threaded_record_writer(self, writer):
        while True:
            record = self._read_q.get()
            if record is False:
                return
            writer.write_record(record)

    def write_record(self, record):
        self._read_q.put_nowait(record)

    def start(self):
        self._thread.start()

    def close(self):
        self._read_q.put_nowait(False)
        self._thread.join()


class BaseDatabaseTest(unittest.TestCase):
    def setUp(self):
        self.connection = DatabaseConnection(':memory:')
        self.connection.row_factory = sqlite3.Row


class BaseThreadedDatabaseWriter(BaseDatabaseTest):
    def setUp(self):
        super(BaseThreadedDatabaseWriter, self).setUp()
        self.threads = []
        self.writer = DatabaseRecordWriter(self.connection)

    def start_n_threads(self, n):
        for _ in range(n):
            t = ThreadedRecordWriter(self.writer)
            t.start()
            self.threads.append(t)

    def tearDown(self):
        for t in self.threads:
            t.close()
        super(BaseThreadedDatabaseWriter, self).tearDown()


@unittest.skipIf(sqlite3 is None, "sqlite3 not supported in this python")
class TestMultithreadedDatabaseWriter(BaseThreadedDatabaseWriter):
    def _write_records(self, thread_number, records):
        t = self.threads[thread_number]
        for record in records:
            t.write_record(record)

    def test_bulk_writes_all_succeed(self):
        thread_count = 10
        self.start_n_threads(thread_count)
        for i in range(thread_count):
            self._write_records(
                i,
                [
                    {
                        'command_id': 'command',
                        'event_type': 'API_CALL',
                        'payload': i,
                        'source': 'TEST',
                        'timestamp': 1234,
                    },
                    {
                        'command_id': 'command',
                        'event_type': 'HTTP_REQUEST',
                        'payload': i,
                        'source': 'TEST',
                        'timestamp': 1234,
                    },
                    {
                        'command_id': 'command',
                        'event_type': 'HTTP_RESPONSE',
                        'payload': i,
                        'source': 'TEST',
                        'timestamp': 1234,
                    },
                    {
                        'command_id': 'command',
                        'event_type': 'PARSED_RESPONSE',
                        'payload': i,
                        'source': 'TEST',
                        'timestamp': 1234,
                    },
                ],
            )
        for t in self.threads:
            t.close()
        thread_id_to_request_id = {}
        cursor = self.connection.execute(
            'SELECT request_id, payload FROM records'
        )
        records = 0
        for record in cursor:
            records += 1
            request_id = record['request_id']
            thread_id = record['payload']
            if thread_id not in thread_id_to_request_id:
                thread_id_to_request_id[thread_id] = request_id
            else:
                prior_request_id = thread_id_to_request_id[thread_id]
                self.assertEqual(request_id, prior_request_id)
        self.assertEqual(records, 4 * thread_count)


@unittest.skipIf(sqlite3 is None, "sqlite3 not supported in this python")
class TestDatabaseRecordWriter(BaseDatabaseTest):
    def test_does_create_table(self):
        cursor = self.connection.execute(
            "SELECT COUNT(*) FROM sqlite_master WHERE "
            "type='table' AND name='records'"
        )
        result = cursor.fetchone()
        self.assertEqual(result[0], 1)

    def test_can_write_record(self):
        writer = DatabaseRecordWriter(connection=self.connection)
        known_record_fields = {
            'command_id': 'command',
            'source': 'TEST',
            'event_type': 'foo',
            'payload': {"foo": "bar"},
            'timestamp': 1234,
        }
        writer.write_record(known_record_fields)

        cursor = self.connection.execute("SELECT COUNT(*) FROM records")
        num_records = cursor.fetchone()
        self.assertEqual(num_records[0], 1)

        cursor.execute("SELECT * FROM records")
        record = dict(cursor.fetchone())
        for col_name, row_value in known_record_fields.items():
            # Normally our reader would take care of parsing the JSON from
            # the payload.
            if col_name == 'payload':
                record[col_name] = json.loads(record[col_name])
                self.assertEqual(record[col_name], row_value)

        self.assertTrue('id' in record.keys())
        self.assertTrue('request_id' in record.keys())

    def test_can_write_many_records(self):
        writer = DatabaseRecordWriter(connection=self.connection)
        known_record_fields = {
            'command_id': 'command',
            'source': 'TEST',
            'event_type': 'foo',
            'payload': '',
            'timestamp': 1234,
        }
        records_to_write = 40
        for _ in range(records_to_write):
            writer.write_record(known_record_fields)

        cursor = self.connection.execute("SELECT COUNT(*) FROM records")
        num_records = cursor.fetchone()
        self.assertEqual(num_records[0], records_to_write)


@unittest.skipIf(sqlite3 is None, "sqlite3 not supported in this python")
class TestDatabaseRecordReader(BaseDatabaseTest):
    def _write_sequence_of_records(self, writer, records):
        for record in records:
            writer.write_record(record)

    def test_yields_nothing_if_no_matching_record_id(self):
        reader = DatabaseRecordReader(self.connection)
        records = [record for record in reader.iter_records('fake_id')]
        self.assertEqual(len(records), 0)

    def test_yields_nothing_no_recent_records(self):
        reader = DatabaseRecordReader(self.connection)
        records = [record for record in reader.iter_latest_records()]
        self.assertEqual(len(records), 0)

    def test_can_read_record(self):
        writer = DatabaseRecordWriter(self.connection)
        self._write_sequence_of_records(
            writer,
            [
                {
                    'command_id': 'command a',
                    'source': 'TEST',
                    'event_type': 'foo',
                    'payload': '',
                    'timestamp': 3,
                },
                {
                    'command_id': 'command a',
                    'source': 'TEST',
                    'event_type': 'bar',
                    'payload': '',
                    'timestamp': 1,
                },
                {
                    'command_id': 'command a',
                    'source': 'TEST',
                    'event_type': 'baz',
                    'payload': '',
                    'timestamp': 4,
                },
            ],
        )
        self._write_sequence_of_records(
            writer,
            [
                {
                    'command_id': 'command b',
                    'source': 'TEST',
                    'event_type': 'qux',
                    'payload': '',
                    'timestamp': 2,
                },
                {
                    'command_id': 'command b',
                    'source': 'TEST',
                    'event_type': 'zip',
                    'payload': '',
                    'timestamp': 6,
                },
            ],
        )
        reader = DatabaseRecordReader(self.connection)
        cursor = self.connection.execute(
            'select id from records where event_type = "foo" limit 1'
        )
        identifier = cursor.fetchone()['id']

        # This should select only the three records from writer_a since we
        # are explicitly looking for the records that match the id of the
        # foo event record.
        records = [record for record in reader.iter_records(identifier)]
        self.assertEqual(len(records), 3)
        for record in records:
            record_id = record['id']
            self.assertEqual(record_id, identifier)

    def test_can_read_most_recent_records(self):
        writer = DatabaseRecordWriter(self.connection)
        self._write_sequence_of_records(
            writer,
            [
                {
                    'command_id': 'command a',
                    'source': 'TEST',
                    'event_type': 'foo',
                    'payload': '',
                    'timestamp': 3,
                },
                {
                    'command_id': 'command a',
                    'source': 'TEST',
                    'event_type': 'bar',
                    'payload': '',
                    'timestamp': 1,
                },
            ],
        )
        self._write_sequence_of_records(
            writer,
            [
                {
                    'command_id': 'command b',
                    'source': 'TEST',
                    'event_type': 'baz',
                    'payload': '',
                    'timestamp': 2,
                }
            ],
        )

        # Since the foo and bar events were written by the writer_a they all
        # share an id. foo was written at time 3 which makes it the most
        # recent, so when we call get_latest_records we should get the
        # foo and bar records only.
        reader = DatabaseRecordReader(self.connection)
        records = set(
            [record['event_type'] for record in reader.iter_latest_records()]
        )
        self.assertEqual(set(['foo', 'bar']), records)


class TestDatabaseHistoryHandler(unittest.TestCase):
    UUID_PATTERN = re.compile(
        '^[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$', re.I
    )

    def setUp(self):
        self.db = DatabaseConnection(':memory:')
        self.writer = DatabaseRecordWriter(connection=self.db)
        self.record_builder = RecordBuilder()
        self.handler = DatabaseHistoryHandler(
            writer=self.writer, record_builder=self.record_builder
        )

    def _get_last_record(self):
        record = self.db.execute('SELECT * FROM records').fetchone()
        return record

    def _assert_expected_event_type(self, source, record):
        self.assertEqual(source, record[3])

    def _assert_expected_payload(self, source, record):
        loaded_payload = json.loads(record[-1])
        self.assertEqual(source, loaded_payload)

    def _assert_expected_source(self, source, record):
        self.assertEqual(source, record[2])

    def _assert_has_request_id(self, record):
        identifier = record[1]
        self.assertTrue(self.UUID_PATTERN.match(identifier))

    def _assert_record_has_command_id(self, record):
        identifier = record[0]
        self.assertTrue(self.UUID_PATTERN.match(identifier))

    def test_does_emit_write_record(self):
        self.handler.emit('event_type', 'payload', 'source')
        record = self._get_last_record()
        self._assert_record_has_command_id(record)
        self._assert_expected_event_type('event_type', record)
        self._assert_expected_payload('payload', record)
        self._assert_expected_source('source', record)

    def test_can_emit_write_record_with_structure(self):
        payload = {'foo': 'bar'}
        self.handler.emit('event_type', payload, 'source')
        record = self._get_last_record()
        self._assert_record_has_command_id(record)
        self._assert_expected_event_type('event_type', record)
        self._assert_expected_payload(payload, record)
        self._assert_expected_source('source', record)

    def test_can_emit_cli_version_record(self):
        # CLI_VERSION records have a list of strings payload
        payload = 'foobarbaz'
        self.handler.emit('CLI_VERSION', payload, 'CLI')
        record = self._get_last_record()
        self._assert_record_has_command_id(record)
        self._assert_expected_event_type('CLI_VERSION', record)
        self._assert_expected_payload(payload, record)
        self._assert_expected_source('CLI', record)

    def test_can_emit_cli_arguments_record(self):
        # CLI_ARGUMENTS records have a list of strings payload
        payload = ['foo', 'bar', 'baz']
        self.handler.emit('CLI_ARGUMENTS', payload, 'CLI')
        record = self._get_last_record()
        self._assert_record_has_command_id(record)
        self._assert_expected_event_type('CLI_ARGUMENTS', record)
        self._assert_expected_payload(payload, record)
        self._assert_expected_source('CLI', record)

    def test_can_emit_api_call_record(self):
        # API_CALL records have a dictionary based payload
        payload = {'service': 's3', 'operation': 'ListBuckets', 'params': {}}
        self.handler.emit('API_CALL', payload, 'BOTOCORE')
        record = self._get_last_record()
        self._assert_record_has_command_id(record)
        self._assert_has_request_id(record)
        self._assert_expected_event_type('API_CALL', record)
        self._assert_expected_payload(payload, record)
        self._assert_expected_source('BOTOCORE', record)

    def test_can_emit_api_call_record_with_binary_param(self):
        # API_CALL records have a dictionary based payload
        payload = {
            'service': 'lambda',
            'operation': 'CreateFunction',
            'params': {
                "FunctionName": "Name",
                "Handler": "mod.fn",
                "Role": "foobar",
                "Runtime": "python3",
                "Code": {"ZipFile": b'zipfile binary content \xfe\xed'},
            },
        }
        self.handler.emit('API_CALL', payload, 'BOTOCORE')
        record = self._get_last_record()
        parsed_payload = payload.copy()
        parsed_payload['params']['Code']['ZipFile'] = '<Byte sequence>'
        self._assert_record_has_command_id(record)
        self._assert_has_request_id(record)
        self._assert_expected_event_type('API_CALL', record)
        self._assert_expected_payload(parsed_payload, record)
        self._assert_expected_source('BOTOCORE', record)

    def test_can_emit_http_request_record(self):
        # HTTP_REQUEST records have have their entire body field as a binary
        # blob, however it will all be utf-8 valid since the binary fields
        # from the api call will have been b64 encoded.
        payload = {
            'url': (
                'https://lambda.us-west-2.amazonaws.com/2015-03-31/'
                'functions'
            ),
            'method': 'POST',
            'headers': CaseInsensitiveDict({'foo': 'bar'}),
            'body': b'body with no invalid utf-8 bytes in it',
            'streaming': False,
        }
        self.handler.emit('HTTP_REQUEST', payload, 'BOTOCORE')
        record = self._get_last_record()
        parsed_payload = payload.copy()
        parsed_payload['headers'] = dict(parsed_payload['headers'])
        parsed_payload['body'] = 'body with no invalid utf-8 bytes in it'
        self._assert_record_has_command_id(record)
        self._assert_expected_event_type('HTTP_REQUEST', record)
        self._assert_expected_payload(parsed_payload, record)
        self._assert_expected_source('BOTOCORE', record)

    def test_can_emit_http_response_record(self):
        # HTTP_RESPONSE also contains a binary response in its body, but it
        # will not contain any non-unicode characters
        payload = {
            'status_code': 200,
            'headers': CaseInsensitiveDict({'foo': 'bar'}),
            'body': b'body with no invalid utf-8 bytes in it',
            'streaming': False,
        }
        self.handler.emit('HTTP_RESPONSE', payload, 'BOTOCORE')
        record = self._get_last_record()
        parsed_payload = payload.copy()
        parsed_payload['headers'] = dict(parsed_payload['headers'])
        parsed_payload['body'] = 'body with no invalid utf-8 bytes in it'
        self._assert_record_has_command_id(record)
        self._assert_expected_event_type('HTTP_RESPONSE', record)
        self._assert_expected_payload(parsed_payload, record)
        self._assert_expected_source('BOTOCORE', record)

    def test_can_emit_parsed_response_record(self):
        payload = {
            "Count": 1,
            "Items": [{"strkey": {"S": "string"}}],
            "ScannedCount": 1,
            "ConsumedCapacity": None,
        }
        self.handler.emit('PARSED_RESPONSE', payload, 'BOTOCORE')
        record = self._get_last_record()
        self._assert_record_has_command_id(record)
        self._assert_expected_event_type('PARSED_RESPONSE', record)
        self._assert_expected_payload(payload, record)
        self._assert_expected_source('BOTOCORE', record)

    def test_can_emit_parsed_response_record_with_binary(self):
        # PARSED_RESPONSE can also contain raw bytes
        payload = {
            "Count": 1,
            "Items": [{"bitkey": {"B": b"binary data \xfe\xed"}}],
            "ScannedCount": 1,
            "ConsumedCapacity": None,
        }
        self.handler.emit('PARSED_RESPONSE', payload, 'BOTOCORE')
        record = self._get_last_record()
        parsed_payload = payload.copy()
        parsed_payload['Items'][0]['bitkey']['B'] = "<Byte sequence>"
        self._assert_record_has_command_id(record)
        self._assert_expected_event_type('PARSED_RESPONSE', record)
        self._assert_expected_payload(payload, record)
        self._assert_expected_source('BOTOCORE', record)

    def test_does_not_mutate_dict(self):
        payload = {"bitkey": b"binary data \xfe\xed"}
        copy_payload = payload.copy()
        self.handler.emit('test', payload, 'BOTOCORE')
        self.assertEqual(payload, copy_payload)

    def test_does_not_mutate_list(self):
        payload = ['non binary data', b"binary data \xfe\xed"]
        copy_payload = list(payload)
        self.handler.emit('test', payload, 'BOTOCORE')
        self.assertEqual(payload, copy_payload)



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import time
from datetime import datetime, timedelta

from botocore.session import Session
from botocore.stub import Stubber
from dateutil import tz

from awscli.compat import StringIO
from awscli.customizations.logs.tail import (
    DetailedLogEventsFormatter,
    FollowLogEventsGenerator,
    NoFollowLogEventsGenerator,
    PrettyJSONLogEventsFormatter,
    ShortLogEventsFormatter,
    TimestampUtils,
)
from awscli.testutils import mock, unittest


class BaseLogEventsFormatterTest(unittest.TestCase):
    FORMATTER_CLS = None

    def setUp(self):
        self.log_event = {
            'timestamp': datetime(2018, 1, 1, 0, 29, 43, 79060, tz.tzutc()),
            'logStreamName': 'stream_name',
            'message': 'my message',
        }
        self.output = StringIO()

    def assert_formatted_display(self, expected_msg, **init_kwargs):
        self.FORMATTER_CLS(self.output, **init_kwargs).display_log_event(
            self.log_event
        )
        self.assertEqual(expected_msg, self.output.getvalue())


class TestShortLogEventsFormatter(BaseLogEventsFormatterTest):
    FORMATTER_CLS = ShortLogEventsFormatter

    def test_display(self):
        self.assert_formatted_display(
            '\x1b[32m2018-01-01T00:29:43\x1b[0m my message\n',
        )

    def test_display_no_color(self):
        self.assert_formatted_display(
            '2018-01-01T00:29:43 my message\n',
            colorize=False,
        )

    def test_ensures_single_newline_ending(self):
        self.log_event['message'] = self.log_event['message'] + '\n\n'
        self.assert_formatted_display(
            '2018-01-01T00:29:43 my message\n', colorize=False
        )

    def test_handles_unicode(self):
        self.log_event['message'] = self.log_event['message'] + '\u00e9'
        self.assert_formatted_display(
            '2018-01-01T00:29:43 my message\u00e9\n', colorize=False
        )


class TestPrettyJSONLogEventsFormatter(BaseLogEventsFormatterTest):
    FORMATTER_CLS = PrettyJSONLogEventsFormatter

    def test_no_json_is_same_as_detailed_output(self):
        # The messages aren't displayed in color so this is the same output
        # regardless of whether or not you enable color.
        self.assert_formatted_display(
            '\x1b[32m2018-01-01T00:29:43.079060+00:00\x1b[0m '
            '\x1b[36mstream_name\x1b[0m '
            'my message\n'
        )

    def test_has_json_message(self):
        self.log_event['message'] = '{"foo": {"bar": "baz"}}'
        self.assert_formatted_display(
            '2018-01-01T00:29:43.079060+00:00 stream_name \n'
            '{\n'
            '    "foo": {\n'
            '        "bar": "baz"\n'
            '    }\n'
            '}\n',
            colorize=False,
        )


class TestDetailedLogEventsFormatter(BaseLogEventsFormatterTest):
    FORMATTER_CLS = DetailedLogEventsFormatter

    def test_display(self):
        self.assert_formatted_display(
            '\x1b[32m2018-01-01T00:29:43.079060+00:00\x1b[0m '
            '\x1b[36mstream_name\x1b[0m '
            'my message\n'
        )

    def test_display_no_color(self):
        self.assert_formatted_display(
            '2018-01-01T00:29:43.079060+00:00 stream_name my message\n',
            colorize=False,
        )

    def test_ensures_single_newline_ending(self):
        self.log_event['message'] = self.log_event['message'] + '\n\n'
        self.assert_formatted_display(
            '2018-01-01T00:29:43.079060+00:00 stream_name my message\n',
            colorize=False,
        )

    def test_handles_unicode(self):
        self.log_event['message'] = self.log_event['message'] + '\u00e9'
        self.assert_formatted_display(
            '2018-01-01T00:29:43.079060+00:00 stream_name my message\u00e9\n',
            colorize=False,
        )

    def test_display_with_zero_microseconds(self):
        self.log_event['timestamp'] = datetime(
            2018, 1, 1, 0, 29, 43, 0, tz.tzutc()
        )
        DetailedLogEventsFormatter(self.output).display_log_event(
            self.log_event
        )
        self.assertEqual(
            '\x1b[32m2018-01-01T00:29:43.000000+00:00\x1b[0m '
            '\x1b[36mstream_name\x1b[0m '
            'my message\n',
            self.output.getvalue(),
        )


class TestTimestampUtils(unittest.TestCase):
    def setUp(self):
        self.mock_now = mock.Mock()
        self.set_now()
        self.timestamp_utils = TimestampUtils(self.mock_now)

    def set_now(self, year=1970, month=1, day=1, hour=0, minute=0, sec=0):
        self.now = datetime(
            year, month, day, hour, minute, sec, tzinfo=tz.tzutc()
        )
        self.mock_now.return_value = self.now

    def test_to_epoch_absolute_timezone_unaware(self):
        self.assertEqual(
            self.timestamp_utils.to_epoch_millis('1970-01-01T00:00:01.000000'),
            1000,
        )

    def test_to_epoch_absolute_timezone_aware(self):
        self.assertEqual(
            self.timestamp_utils.to_epoch_millis(
                '1970-01-01T00:00:01.000000-01:00'
            ),
            3601000,
        )

    def test_to_epoch_relative_second(self):
        self.set_now(sec=2)
        self.assertEqual(self.timestamp_utils.to_epoch_millis('1s'), 1000)

    def test_to_epoch_relative_multiple_seconds(self):
        self.set_now(sec=5)
        self.assertEqual(self.timestamp_utils.to_epoch_millis('2s'), 3000)

    def test_to_epoch_relative_minute(self):
        self.set_now(minute=2)
        self.assertEqual(self.timestamp_utils.to_epoch_millis('1m'), 60 * 1000)

    def test_to_epoch_relative_hour(self):
        self.set_now(hour=2)
        self.assertEqual(
            self.timestamp_utils.to_epoch_millis('1h'), (60 * 60) * 1000
        )

    def test_to_epoch_relative_day(self):
        self.set_now(day=3)  # 1970-01-03
        self.assertEqual(
            self.timestamp_utils.to_epoch_millis('1d'),
            (24 * 60 * 60) * 1000,  # 1970-01-02
        )

    def test_to_epoch_relative_week(self):
        self.set_now(day=9)  # 1970-01-08
        self.assertEqual(
            self.timestamp_utils.to_epoch_millis('1w'),
            (24 * 60 * 60) * 1000,  # 1970-01-02
        )

    def test_to_datetime(self):
        self.assertEqual(
            self.timestamp_utils.to_datetime(1000),
            datetime(1970, 1, 1, 0, 0, 1, tzinfo=tz.tzutc()),
        )


class BaseLogEventsGeneratorTest(unittest.TestCase):
    def setUp(self):
        self.session = Session()
        self.client = self.session.create_client(
            'logs', region_name='us-west-2'
        )
        self.stubber = Stubber(self.client)
        self.group_name = 'groupName'
        self.start = '1970-01-01T00:00:01.000000'
        self.expected_start_as_milli_epoch = 1000
        self.filter_pattern = 'mypattern'
        self.log_stream_names = ['foo-stream', 'bar-stream']
        self.log_stream_name_prefix = 'foo'
        self.log_timestamp = 1000
        self.expected_log_timestamp_as_datetime = datetime(
            1970, 1, 1, 0, 0, 1, tzinfo=tz.tzutc()
        )

    def get_event(self, event_id, event_message, timestamp=None):
        if timestamp is None:
            timestamp = self.log_timestamp
        return {
            'eventId': event_id,
            'message': event_message,
            'timestamp': timestamp,
            'ingestionTime': self.log_timestamp,
        }

    def get_expected_event(self, event_id, event_message, add_seconds=0):
        return {
            'eventId': event_id,
            'message': event_message,
            'timestamp': self.expected_log_timestamp_as_datetime
            + timedelta(seconds=add_seconds),
            'ingestionTime': self.expected_log_timestamp_as_datetime,
        }


class TestNoFollowLogEventsGenerator(BaseLogEventsGeneratorTest):
    def test_iter_log_events(self):
        logs_generator = NoFollowLogEventsGenerator(
            self.client, TimestampUtils()
        )

        self.stubber.add_response(
            'filter_log_events',
            service_response={
                'events': [
                    self.get_event('event-1', 'event-1-message'),
                    self.get_event('event-2', 'event-2-message'),
                ],
                'nextToken': 'token',
            },
            expected_params={
                'logGroupName': self.group_name,
                'interleaved': True,
                'startTime': self.expected_start_as_milli_epoch,
                'filterPattern': self.filter_pattern,
                'logStreamNames': self.log_stream_names,
                'logStreamNamePrefix': self.log_stream_name_prefix,
            },
        )
        # Add a new page that has no more results
        self.stubber.add_response(
            'filter_log_events',
            service_response={
                'events': [
                    self.get_event('event-3', 'event-3-message'),
                ],
            },
            expected_params={
                'logGroupName': self.group_name,
                'interleaved': True,
                'startTime': self.expected_start_as_milli_epoch,
                'filterPattern': self.filter_pattern,
                'logStreamNames': self.log_stream_names,
                'logStreamNamePrefix': self.log_stream_name_prefix,
                'nextToken': 'token',
            },
        )
        with self.stubber:
            log_events_iter = logs_generator.iter_log_events(
                self.group_name,
                self.start,
                self.filter_pattern,
                self.log_stream_names,
                self.log_stream_name_prefix,
            )
            actual_log_events = [event for event in log_events_iter]
        self.assertEqual(
            actual_log_events,
            [
                self.get_expected_event('event-1', 'event-1-message'),
                self.get_expected_event('event-2', 'event-2-message'),
                self.get_expected_event('event-3', 'event-3-message'),
            ],
        )


class TestFollowLogEventsGenerator(BaseLogEventsGeneratorTest):
    def setUp(self):
        super(TestFollowLogEventsGenerator, self).setUp()
        self.mock_sleep = mock.Mock(time.sleep)
        self.logs_generator = FollowLogEventsGenerator(
            self.client, TimestampUtils(), self.mock_sleep
        )

    def test_iter_log_events(self):
        self.stubber.add_response(
            'filter_log_events',
            service_response={
                'events': [
                    self.get_event('event-1', 'event-1-message'),
                    self.get_event('event-2', 'event-2-message'),
                ],
                'nextToken': 'token',
            },
            expected_params={
                'logGroupName': self.group_name,
                'interleaved': True,
                'startTime': self.expected_start_as_milli_epoch,
                'filterPattern': self.filter_pattern,
                'logStreamNames': self.log_stream_names,
                'logStreamNamePrefix': self.log_stream_name_prefix,
            },
        )
        # Add a new page that has no nextToken
        self.stubber.add_response(
            'filter_log_events',
            service_response={
                'events': [
                    self.get_event('event-3', 'event-3-message'),
                ],
            },
            expected_params={
                'logGroupName': self.group_name,
                'interleaved': True,
                'startTime': self.expected_start_as_milli_epoch,
                'filterPattern': self.filter_pattern,
                'logStreamNames': self.log_stream_names,
                'logStreamNamePrefix': self.log_stream_name_prefix,
                'nextToken': 'token',
            },
        )
        self.mock_sleep.side_effect = KeyboardInterrupt()
        with self.stubber:
            log_events_iter = self.logs_generator.iter_log_events(
                self.group_name,
                self.start,
                self.filter_pattern,
                self.log_stream_names,
                self.log_stream_name_prefix,
            )
            actual_log_events = [event for event in log_events_iter]
        self.mock_sleep.assert_called_once_with(5)
        self.assertEqual(
            actual_log_events,
            [
                self.get_expected_event('event-1', 'event-1-message'),
                self.get_expected_event('event-2', 'event-2-message'),
                self.get_expected_event('event-3', 'event-3-message'),
            ],
        )

    def test_polls_with_last_next_token(self):
        self.stubber.add_response(
            'filter_log_events',
            service_response={
                'events': [
                    self.get_event('event-1', 'event-1-message', 1000),
                ],
                'nextToken': 'token',
            },
            expected_params={
                'logGroupName': self.group_name,
                'interleaved': True,
            },
        )
        # Add a new page that has no nextToken
        self.stubber.add_response(
            'filter_log_events',
            service_response={
                'events': [
                    self.get_event('event-2', 'event-2-message', 2000),
                ],
            },
            expected_params={
                'logGroupName': self.group_name,
                'interleaved': True,
                'nextToken': 'token',
            },
        )

        # Because there is no token for the last page, it should remove
        # token from the kwargs and update startTime with the max
        # timestamp from the prvious response events
        self.stubber.add_response(
            'filter_log_events',
            service_response={
                'events': [
                    self.get_event('event-3', 'event-3-message', 3000),
                ],
            },
            expected_params={
                'logGroupName': self.group_name,
                'interleaved': True,
                'startTime': 2000,
            },
        )
        self.mock_sleep.side_effect = [None, KeyboardInterrupt()]
        with self.stubber:
            log_events_iter = self.logs_generator.iter_log_events(
                self.group_name
            )
            actual_log_events = [event for event in log_events_iter]
        self.mock_sleep.assert_has_calls([mock.call(5), mock.call(5)])
        self.assertEqual(
            actual_log_events,
            [
                self.get_expected_event('event-1', 'event-1-message'),
                self.get_expected_event('event-2', 'event-2-message', 1),
                self.get_expected_event('event-3', 'event-3-message', 2),
            ],
        )

    def test_iter_log_events_filters_empty_events_list(self):
        self.stubber.add_response(
            'filter_log_events',
            service_response={
                'events': [],
            },
            expected_params={
                'logGroupName': self.group_name,
                'interleaved': True,
            },
        )
        # Add a new page that has events
        # It should make a call with the same parameters as the
        # previous one
        self.stubber.add_response(
            'filter_log_events',
            service_response={
                'events': [
                    self.get_event('event-1', 'event-1-message'),
                    self.get_event('event-2', 'event-2-message'),
                    self.get_event('event-3', 'event-3-message'),
                ],
            },
            expected_params={
                'logGroupName': self.group_name,
                'interleaved': True,
            },
        )
        self.mock_sleep.side_effect = [None, KeyboardInterrupt()]
        with self.stubber:
            log_events_iter = self.logs_generator.iter_log_events(
                self.group_name
            )
            actual_log_events = [event for event in log_events_iter]
        self.mock_sleep.assert_has_calls([mock.call(5), mock.call(5)])
        self.assertEqual(
            actual_log_events,
            [
                self.get_expected_event('event-1', 'event-1-message'),
                self.get_expected_event('event-2', 'event-2-message'),
                self.get_expected_event('event-3', 'event-3-message'),
            ],
        )

    def test_iter_log_events_filters_old_events(self):
        self.stubber.add_response(
            'filter_log_events',
            service_response={
                'events': [
                    self.get_event('event-1', 'event-1-message'),
                    self.get_event('event-2', 'event-2-message'),
                ],
            },
            expected_params={
                'logGroupName': self.group_name,
                'interleaved': True,
            },
        )
        # Add a new page that has no nextToken
        # It should update startTime with the max timestamp
        # from the prvious response events
        self.stubber.add_response(
            'filter_log_events',
            service_response={
                'events': [
                    self.get_event('event-1', 'event-1-message'),
                    self.get_event('event-2', 'event-2-message'),
                    self.get_event('event-3', 'event-3-message'),
                ],
            },
            expected_params={
                'logGroupName': self.group_name,
                'interleaved': True,
                'startTime': self.get_event('event-2', 'event-2-message')[
                    'timestamp'
                ],
            },
        )
        self.mock_sleep.side_effect = [None, KeyboardInterrupt()]
        with self.stubber:
            log_events_iter = self.logs_generator.iter_log_events(
                self.group_name
            )
            actual_log_events = [event for event in log_events_iter]
        self.mock_sleep.assert_has_calls([mock.call(5), mock.call(5)])
        self.assertEqual(
            actual_log_events,
            [
                self.get_expected_event('event-1', 'event-1-message'),
                self.get_expected_event('event-2', 'event-2-message'),
                self.get_expected_event('event-3', 'event-3-message'),
            ],
        )



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/

from unittest.mock import ANY, Mock, patch

import pytest

from awscli.customizations.ecs import inject_commands
from awscli.customizations.ecs.monitorexpressgatewayservice import (
    ECSMonitorExpressGatewayService,
)


@pytest.fixture
def mock_watcher_class():
    """Fixture that provides a mock watcher class."""
    return Mock()


@pytest.fixture
def mock_session():
    """Fixture that provides a mock session."""
    return Mock()


@pytest.fixture
def command(mock_session, mock_watcher_class):
    """Fixture that provides an ECSMonitorExpressGatewayService command."""
    return ECSMonitorExpressGatewayService(
        mock_session, watcher_class=mock_watcher_class
    )


@pytest.fixture
def command_with_mock_session(mock_session, mock_watcher_class):
    """Fixture that provides command with mock session and client configured."""
    client = Mock()
    mock_session.create_client.return_value = client
    command = ECSMonitorExpressGatewayService(
        mock_session, watcher_class=mock_watcher_class
    )
    return command


class TestECSMonitorExpressGatewayService:
    def test_init(self, command):
        assert command.name == 'monitor-express-gateway-service'
        assert command.DESCRIPTION.startswith('Monitors the progress')

    def test_add_arguments(self, command):
        command._build_arg_table()
        arg_table = command.arg_table

        assert 'service-arn' in arg_table
        assert 'resource-view' in arg_table
        assert 'timeout' in arg_table
        assert 'mode' in arg_table

        # Verify resource-view argument has correct choices
        resource_view_arg = arg_table['resource-view']
        assert resource_view_arg.choices == ['RESOURCE', 'DEPLOYMENT']

        # Verify mode argument has correct choices
        mode_arg = arg_table['mode']
        assert mode_arg.choices == ['INTERACTIVE', 'TEXT-ONLY']

    @patch('sys.stdout.isatty', return_value=False)
    def test_run_main_with_text_only_mode(
        self, mock_isatty, command_with_mock_session, mock_watcher_class
    ):
        command = command_with_mock_session
        mock_watcher = Mock()
        mock_watcher_class.return_value = mock_watcher

        parsed_args = Mock(
            service_arn='arn:aws:ecs:us-west-2:123456789012:service/cluster/service',
            resource_view='RESOURCE',
            timeout=30,
            mode='TEXT-ONLY',
        )
        parsed_globals = Mock(
            region='us-west-2',
            endpoint_url=None,
            verify_ssl=True,
            color='off',
        )

        command._run_main(parsed_args, parsed_globals)

        # Verify watcher was created with correct parameters (positional)
        mock_watcher_class.assert_called_once_with(
            ANY,
            parsed_args.service_arn,
            'RESOURCE',
            'TEXT-ONLY',
            timeout_minutes=30,
            use_color=False,
        )

        # Verify watcher was executed
        mock_watcher.exec.assert_called_once()

    @patch('sys.stdout.isatty', return_value=True)
    def test_run_main_with_interactive_mode(
        self, mock_isatty, command_with_mock_session, mock_watcher_class
    ):
        command = command_with_mock_session
        mock_watcher = Mock()
        mock_watcher_class.return_value = mock_watcher

        parsed_args = Mock(
            service_arn='arn:aws:ecs:us-west-2:123456789012:service/cluster/service',
            resource_view='DEPLOYMENT',
            timeout=60,
            mode='INTERACTIVE',
        )
        parsed_globals = Mock(
            region='us-west-2',
            endpoint_url=None,
            verify_ssl=True,
            color='auto',
        )

        command._run_main(parsed_args, parsed_globals)

        # Verify watcher was created with correct mode
        mock_watcher_class.assert_called_once_with(
            ANY,
            parsed_args.service_arn,
            'DEPLOYMENT',
            'INTERACTIVE',
            timeout_minutes=60,
            use_color=True,
        )

    @patch('sys.stdout.isatty', return_value=True)
    def test_run_main_auto_mode_with_tty(
        self, mock_isatty, command_with_mock_session, mock_watcher_class
    ):
        command = command_with_mock_session
        mock_watcher = Mock()
        mock_watcher_class.return_value = mock_watcher

        parsed_args = Mock(
            service_arn='arn:aws:ecs:us-west-2:123456789012:service/cluster/service',
            resource_view='RESOURCE',
            timeout=30,
            mode=None,  # Auto mode
        )
        parsed_globals = Mock(
            region='us-west-2',
            endpoint_url=None,
            verify_ssl=True,
            color='auto',
        )

        command._run_main(parsed_args, parsed_globals)

        # When mode is None and TTY is available, should use INTERACTIVE
        args = mock_watcher_class.call_args[0]
        assert args[3] == 'INTERACTIVE'

    @patch('sys.stdout.isatty', return_value=False)
    def test_run_main_auto_mode_without_tty(
        self, mock_isatty, command_with_mock_session, mock_watcher_class
    ):
        command = command_with_mock_session
        mock_watcher = Mock()
        mock_watcher_class.return_value = mock_watcher

        parsed_args = Mock(
            service_arn='arn:aws:ecs:us-west-2:123456789012:service/cluster/service',
            resource_view='RESOURCE',
            timeout=30,
            mode=None,  # Auto mode
        )
        parsed_globals = Mock(
            region='us-west-2',
            endpoint_url=None,
            verify_ssl=True,
            color='auto',
        )

        command._run_main(parsed_args, parsed_globals)

        # When mode is None and TTY is not available, should use TEXT-ONLY
        args = mock_watcher_class.call_args[0]
        assert args[3] == 'TEXT-ONLY'

    @patch('sys.stdout.isatty', return_value=False)
    def test_run_main_with_color_on(
        self, mock_isatty, command_with_mock_session, mock_watcher_class
    ):
        command = command_with_mock_session
        mock_watcher = Mock()
        mock_watcher_class.return_value = mock_watcher

        parsed_args = Mock(
            service_arn='arn:aws:ecs:us-west-2:123456789012:service/cluster/service',
            resource_view='RESOURCE',
            timeout=30,
            mode='TEXT-ONLY',
        )
        parsed_globals = Mock(
            region='us-west-2',
            endpoint_url=None,
            verify_ssl=True,
            color='on',
        )

        command._run_main(parsed_args, parsed_globals)

        # Verify color setting is True when color='on'
        call_kwargs = mock_watcher_class.call_args[1]
        assert call_kwargs['use_color'] is True

    @patch('sys.stdout.isatty', return_value=False)
    def test_run_main_creates_ecs_client(
        self,
        mock_isatty,
        mock_session,
        command_with_mock_session,
        mock_watcher_class,
    ):
        command = command_with_mock_session
        mock_watcher = Mock()
        mock_watcher_class.return_value = mock_watcher

        parsed_args = Mock(
            service_arn='arn:aws:ecs:us-west-2:123456789012:service/cluster/service',
            resource_view='RESOURCE',
            timeout=30,
            mode='TEXT-ONLY',
        )
        parsed_globals = Mock(
            region='us-west-2',
            endpoint_url=None,
            verify_ssl=True,
            color='off',
        )

        command._run_main(parsed_args, parsed_globals)

        # Verify ECS client was created with correct parameters
        mock_session.create_client.assert_called_once_with(
            'ecs',
            region_name='us-west-2',
            endpoint_url=None,
            verify=True,
        )

        # Verify client was passed to watcher
        args = mock_watcher_class.call_args[0]
        assert args[0] is not None  # Client was created and passed

    @patch('sys.stdout.isatty', return_value=False)
    def test_run_main_with_default_resource_view(
        self, mock_isatty, command_with_mock_session, mock_watcher_class
    ):
        command = command_with_mock_session
        mock_watcher = Mock()
        mock_watcher_class.return_value = mock_watcher

        parsed_args = Mock(
            service_arn='arn:aws:ecs:us-west-2:123456789012:service/cluster/service',
            resource_view=None,  # Not specified, should use default
            timeout=30,
            mode='TEXT-ONLY',
        )
        parsed_globals = Mock(
            region='us-west-2',
            endpoint_url=None,
            verify_ssl=True,
            color='off',
        )

        command._run_main(parsed_args, parsed_globals)

        # Verify default resource view is passed
        args = mock_watcher_class.call_args[0]
        assert args[2] is None  # Resource view is passed as-is


class TestCommandRegistration:
    def test_inject_commands_registers_monitor_command(self, mock_session):
        command_table = {}

        inject_commands(command_table, mock_session)

        # Verify monitor command is registered
        assert 'monitor-express-gateway-service' in command_table
        command = command_table['monitor-express-gateway-service']
        assert isinstance(command, ECSMonitorExpressGatewayService)



# Copyright 2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import json
import os

import awscrt.io
import pytest
from prompt_toolkit.keys import Keys

from awscli.autocomplete import db, filters, generator, parser
from awscli.autocomplete.local import indexer, model
from awscli.autocomplete.main import create_autocompleter
from awscli.autoprompt.factory import PromptToolkitFactory
from awscli.autoprompt.history import HistoryDriver
from awscli.autoprompt.prompttoolkit import (
    PromptToolkitCompleter,
    PromptToolkitPrompter,
)
from awscli.clidriver import create_clidriver
from awscli.testutils import FileCreator, cd, mock
from tests import PromptToolkitAppRunner


def _cloudwatch_only_command_table(command_table, **kwargs):
    for key in list(command_table):
        if key != 'cloudwatch':
            del command_table[key]


def _generate_index_if_needed(db_connection):
    if not _index_has_been_generated(db_connection):
        # This will eventually be moved into some utility function.
        index_generator = generator.IndexGenerator(
            [indexer.ModelIndexer(db_connection)],
        )
        driver = create_clidriver()
        driver.session.register(
            'building-command-table.main', _cloudwatch_only_command_table
        )
        index_generator.generate_index(driver)


def _index_has_been_generated(db_connection):
    result = db_connection.execute(
        "SELECT name FROM sqlite_master WHERE type='table' "
        "AND name='command_table';"
    )
    return any(result.fetchall())


@pytest.fixture()
def files():
    files = FileCreator()
    yield files
    files.remove_all()


@pytest.fixture(scope='module')
def model_index():
    index_filename = 'file::memory:?cache=shared'
    db_connection = db.DatabaseConnection(index_filename)
    _generate_index_if_needed(db_connection)
    yield index_filename
    db_connection.close()


@pytest.fixture
def history_file(files):
    history = {
        'version': 1,
        'commands': [
            'accessanalyzer update-findings',
            'cloudwatch describe-alarms',
            's3 ls',
        ],
    }
    return files.create_file('prompt_history.json', json.dumps(history))


@pytest.fixture
def prompter(model_index, history_file, ptk_app_session):
    cli_parser = parser.CLIParser(model.ModelIndex(model_index))
    completion_source = create_autocompleter(
        model_index, response_filter=filters.fuzzy_filter
    )
    completer = PromptToolkitCompleter(completion_source)
    history_driver = HistoryDriver(history_file)
    driver = create_clidriver()
    factory = PromptToolkitFactory(completer, history_driver=history_driver)
    return PromptToolkitPrompter(
        completion_source=completion_source,
        driver=driver,
        factory=factory,
        cli_parser=cli_parser,
        output=ptk_app_session.output,
        app_input=ptk_app_session.input,
    )


@pytest.fixture
def app_runner(prompter):
    return PromptToolkitAppRunner(app=prompter.app, pre_run=prompter.pre_run)


class BasicPromptToolkitTest:
    def assert_current_buffer(self, app, expected_current_buffer_name):
        assert app.current_buffer.name == expected_current_buffer_name

    def assert_current_buffer_text(self, app, expected):
        assert app.current_buffer.text == expected

    def assert_buffer_is_visible(self, app, buffer_name):
        assert buffer_name in self.get_all_visible_buffers(app)

    def assert_buffer_is_not_visible(self, app, buffer_name):
        assert buffer_name not in self.get_all_visible_buffers(app)

    def get_all_visible_buffers(self, app):
        return [
            window.content.buffer.name
            for window in app.layout.visible_windows
            if hasattr(window.content, 'buffer')
        ]


class TestPromptToolkitPrompterBuffer:
    """This set of tests tests that we set the buffer (aka "construction zone")
    correctly. Some of these tests test against specific edge cases that have
    previously been known to produce unexpected behavior.

    """

    @pytest.mark.parametrize(
        'args,expected_input_buffer_text',
        [
            (['cloudwatch', 'fake'], 'cloudwatch fake '),
            (['cloudwatch'], 'cloudwatch '),
            (['cloud'], 'cloud'),
            (
                ['s3', 'mv', '/path/to/file/1', 's3://path/to/file/2'],
                's3 mv /path/to/file/1 s3://path/to/file/2 ',
            ),
            (['s3', 'ls'], 's3 ls '),
            (['cloudwatch', 'desc'], 'cloudwatch desc'),
            (['s3', 'ls '], 's3 ls '),
            (['cloudwatch', 'fake', '--output'], 'cloudwatch fake --output '),
            (
                ['cloudwatch', 'describe-alarms', '--output'],
                'cloudwatch describe-alarms --output ',
            ),
            ([' '], ' '),
        ],
    )
    def test_input_buffer_initialization(
        self, prompter, args, expected_input_buffer_text
    ):
        prompter.args = args
        app_runner = PromptToolkitAppRunner(
            app=prompter.app, pre_run=prompter.pre_run
        )
        with app_runner.run_app_in_thread():
            actual_input_text = prompter.input_buffer.document.text
            assert actual_input_text == expected_input_buffer_text

    def test_handle_args_with_spaces(self, app_runner, prompter):
        original_args = ['iam', 'create-role', '--description', 'With spaces']
        prompter.args = original_args
        with app_runner.run_app_in_thread(
            target=prompter.prompt_for_args, args=(original_args,)
        ) as ctx:
            assert prompter.input_buffer.document.text == (
                "iam create-role --description 'With spaces' "
            )
        assert ctx.return_value == original_args


class TestPromptToolkitDocBuffer(BasicPromptToolkitTest):
    def assert_doc_panel_is_visible(self, app):
        assert app.show_doc

    def assert_doc_panel_is_not_visible(self, app):
        assert not app.show_doc

    def assert_doc_panel_cursor_position(self, app, expected_row):
        assert (
            app.layout.get_buffer_by_name(
                'doc_buffer'
            ).document.cursor_position_row
            == expected_row
        )

    def test_doc_buffer_not_shown_on_start_and_not_focusable(self, app_runner):
        with app_runner.run_app_in_thread():
            self.assert_doc_panel_is_not_visible(app_runner.app)
            app_runner.feed_input(Keys.F1)
            self.assert_current_buffer(app_runner.app, 'input_buffer')

    def test_doc_buffer_shows_hides_on_F3(self, app_runner):
        with app_runner.run_app_in_thread():
            app_runner.feed_input(Keys.F3)
            self.assert_doc_panel_is_visible(app_runner.app)
            app_runner.feed_input(Keys.F3)
            self.assert_doc_panel_is_not_visible(app_runner.app)
            self.assert_current_buffer(app_runner.app, 'input_buffer')

    def test_doc_buffer_gets_and_removes_focus(self, app_runner):
        with app_runner.run_app_in_thread():
            app_runner.feed_input(Keys.F3)
            self.assert_doc_panel_is_visible(app_runner.app)
            app_runner.feed_input(Keys.F2)
            self.assert_current_buffer(app_runner.app, 'doc_buffer')
            app_runner.feed_input('q')
            self.assert_current_buffer(app_runner.app, 'input_buffer')

    def test_doc_buffer_keeps_position_if_content_dont_change(
        self, prompter, app_runner
    ):
        prompter.args = ['cloudwatch', 'describe-alarms']
        with app_runner.run_app_in_thread():
            # Open the doc panel, focus on it, and go to its top
            app_runner.feed_input(Keys.F3, Keys.F2, 'g')
            self.assert_doc_panel_cursor_position(
                app_runner.app, expected_row=0
            )
            # Move three rows down
            app_runner.feed_input('j', 'j', 'j')
            self.assert_doc_panel_cursor_position(
                app_runner.app, expected_row=3
            )
            # Focus on the input buffer
            app_runner.feed_input('q')
            # Add parameters to the currently inputted command
            app_runner.feed_input('--alarm-names')
            # The doc position should not have moved
            self.assert_doc_panel_cursor_position(
                app_runner.app, expected_row=3
            )

    @pytest.mark.parametrize(
        'args,expected_docs',
        [
            ([' '], 'AWS Command Line Interface'),
            (['fake'], 'AWS Command Line Interface'),
            (['cloudwatch'], 'Amazon CloudWatch'),
            (['cloudwatch', 'fake'], 'Amazon CloudWatch'),
            (['cloudwatch', 'describe-alarms '], 'Retrieves the specified'),
            (
                ['cloudwatch', 'describe-alarms', '--alarm-names'],
                'The names of the alarms',
            ),
        ],
    )
    def test_doc_panel_content(self, prompter, args, expected_docs):
        prompter.args = args
        app_runner = PromptToolkitAppRunner(
            app=prompter.app, pre_run=prompter.pre_run
        )
        with app_runner.run_app_in_thread():
            assert expected_docs in prompter.doc_buffer.document.text


class TestHistoryMode(BasicPromptToolkitTest):
    def assert_history_mode_is_enabled(self, app):
        assert app.current_buffer.history_mode

    def assert_history_mode_is_disabled(self, app):
        assert not app.current_buffer.history_mode

    def assert_selected_history_completion(self, app, expected):
        actual = app.current_buffer.complete_state.current_completion.text
        assert actual == expected

    def test_history_mode_disabled_on_start_and_switched_by_control_R(
        self, app_runner
    ):
        with app_runner.run_app_in_thread():
            self.assert_history_mode_is_disabled(app_runner.app)
            app_runner.feed_input(Keys.ControlR)
            self.assert_history_mode_is_enabled(app_runner.app)
            app_runner.feed_input(Keys.ControlR)
            self.assert_history_mode_is_disabled(app_runner.app)

    def test_choose_and_disable_history_mode_with_enter(
        self, app_runner, prompter
    ):
        prompter.args = ['s3']
        with app_runner.run_app_in_thread():
            self.assert_history_mode_is_disabled(app_runner.app)
            app_runner.feed_input(Keys.ControlR)
            self.assert_history_mode_is_enabled(app_runner.app)
            app_runner.feed_input(Keys.Down)
            self.assert_selected_history_completion(app_runner.app, 's3 ls')
            app_runner.feed_input(Keys.Enter)
            self.assert_history_mode_is_disabled(app_runner.app)
            self.assert_current_buffer(app_runner.app, 'input_buffer')
            self.assert_current_buffer_text(app_runner.app, 's3 ls')

    def test_choose_and_disable_history_mode_with_space(
        self, app_runner, prompter
    ):
        prompter.args = ['s3']
        with app_runner.run_app_in_thread():
            self.assert_history_mode_is_disabled(app_runner.app)
            app_runner.feed_input(Keys.ControlR)
            self.assert_history_mode_is_enabled(app_runner.app)
            app_runner.feed_input(Keys.Down)
            self.assert_selected_history_completion(app_runner.app, 's3 ls')
            app_runner.feed_input(' ')
            self.assert_history_mode_is_disabled(app_runner.app)
            self.assert_current_buffer(app_runner.app, 'input_buffer')
            self.assert_current_buffer_text(app_runner.app, 's3 ls ')


class TestCompletions(BasicPromptToolkitTest):
    def test_service_full_name_shown(self, app_runner, prompter):
        prompter.args = ['c']
        with app_runner.run_app_in_thread():
            app_runner.wait_for_completions_on_current_buffer()
            first_completion = (
                app_runner.app.current_buffer.complete_state.completions[0]
            )
            assert 'Amazon CloudWatch' in first_completion.display_meta_text

    def test_switch_to_multicolumn_mode(self, app_runner, prompter):
        prompter.args = ['cloudwatch']
        with app_runner.run_app_in_thread():
            assert not app_runner.app.multi_column
            app_runner.feed_input(Keys.F4)
            assert app_runner.app.multi_column
            app_runner.feed_input(Keys.F4)
            assert not app_runner.app.multi_column


class TestHelpPanel(BasicPromptToolkitTest):
    def test_help_panel_disabled_on_start_and_appear_on_F1(
        self, app_runner, prompter
    ):
        with app_runner.run_app_in_thread():
            assert not app_runner.app.show_help
            app_runner.feed_input(Keys.F1)
            assert app_runner.app.show_help
            app_runner.feed_input(Keys.F1)
            assert not app_runner.app.show_help

    def test_show_correct_help_panel(self, app_runner, prompter):
        with app_runner.run_app_in_thread():
            app_runner.feed_input(Keys.F1)
            self.assert_buffer_is_visible(app_runner.app, 'help_input')
            self.assert_buffer_is_not_visible(app_runner.app, 'help_doc')
            # Open doc panel and focus on it
            app_runner.feed_input(Keys.F3, Keys.F2)
            # The help for the doc panel should now be visible instead of
            # the help for the main input buffer
            self.assert_buffer_is_visible(app_runner.app, 'help_doc')
            self.assert_buffer_is_not_visible(app_runner.app, 'help_input')

    def test_toolbar_hides_when_help_panel_visible(self, app_runner, prompter):
        with app_runner.run_app_in_thread():
            self.assert_buffer_is_visible(app_runner.app, 'toolbar_input')
            app_runner.feed_input(Keys.F1)
            self.assert_buffer_is_not_visible(app_runner.app, 'toolbar_input')
            self.assert_buffer_is_not_visible(app_runner.app, 'toolbar_doc')


class TestDebugPanel(BasicPromptToolkitTest):
    def test_debug_panel_not_visible_in_non_debug_mode(
        self, app_runner, prompter
    ):
        with app_runner.run_app_in_thread():
            self.assert_buffer_is_not_visible(app_runner.app, 'debug_buffer')

    def test_debug_panel_visible_in_debug_mode(self, app_runner, prompter):
        prompter.app.debug = True
        with app_runner.run_app_in_thread():
            self.assert_buffer_is_visible(app_runner.app, 'debug_buffer')

    def test_open_save_dialog_on_control_s(self, app_runner, prompter):
        prompter.app.debug = True
        with app_runner.run_app_in_thread():
            app_runner.feed_input(Keys.ControlS)
            self.assert_current_buffer_text(app_runner.app, 'prompt_debug.log')

    def test_can_save_log_file(self, app_runner, prompter, files):
        prompter.app.debug = True
        log_file_path = files.full_path('prompt_debug.log')
        with cd(files.rootdir):
            with app_runner.run_app_in_thread():
                app_runner.feed_input(Keys.ControlS)
                app_runner.feed_input(Keys.Enter)
        assert os.path.exists(log_file_path)

    def test_can_switch_focus_between_panels(self, app_runner, prompter):
        prompter.app.debug = True
        with app_runner.run_app_in_thread():
            self.assert_current_buffer(app_runner.app, 'input_buffer')
            app_runner.feed_input(Keys.F2)
            self.assert_current_buffer(app_runner.app, 'debug_buffer')
            app_runner.feed_input(Keys.F2)
            self.assert_current_buffer(app_runner.app, 'input_buffer')

    @mock.patch('awscrt.io.set_log_level')
    def test_debug_mode_does_not_allow_crt_logging(
        self, mock_init_logging, app_runner, prompter
    ):
        args = ['cloudwatch', 'describe-alarms', '--debug']
        with app_runner.run_app_in_thread(
            target=prompter.prompt_for_args, args=(args,)
        ):
            assert app_runner.app.debug
        mock_init_logging.assert_called_with(
            awscrt.io.LogLevel.NoLogs,
        )


class TestOutputPanel(BasicPromptToolkitTest):
    def test_output_panel_not_visible_on_start(self, app_runner):
        with app_runner.run_app_in_thread():
            self.assert_buffer_is_not_visible(app_runner.app, 'output_buffer')

    def test_output_panel_switches_on_F5(self, app_runner):
        with app_runner.run_app_in_thread():
            app_runner.feed_input(Keys.F5)
            self.assert_buffer_is_visible(app_runner.app, 'output_buffer')
            app_runner.feed_input(Keys.F5)
            self.assert_buffer_is_not_visible(app_runner.app, 'output_buffer')

    def test_output_panel_and_doc_panel_can_be_visible_together(
        self, app_runner
    ):
        with app_runner.run_app_in_thread():
            app_runner.feed_input(Keys.F5)
            self.assert_buffer_is_visible(app_runner.app, 'output_buffer')
            self.assert_buffer_is_not_visible(app_runner.app, 'doc_buffer')
            app_runner.feed_input(Keys.F3)
            self.assert_buffer_is_visible(app_runner.app, 'doc_buffer')
            self.assert_buffer_is_visible(app_runner.app, 'output_buffer')

    def test_can_switch_focus_between_panels(self, app_runner):
        with app_runner.run_app_in_thread():
            app_runner.feed_input(Keys.F5, Keys.F3)
            self.assert_current_buffer(app_runner.app, 'input_buffer')
            app_runner.feed_input(Keys.F2)
            self.assert_current_buffer(app_runner.app, 'doc_buffer')
            app_runner.feed_input(Keys.F2)
            self.assert_current_buffer(app_runner.app, 'output_buffer')
            app_runner.feed_input(Keys.F2)
            self.assert_current_buffer(app_runner.app, 'input_buffer')

    def test_can_quit_output_panel(self, app_runner):
        with app_runner.run_app_in_thread():
            app_runner.feed_input(Keys.F5, Keys.F2)
            self.assert_current_buffer(app_runner.app, 'output_buffer')
            app_runner.feed_input('q')
            self.assert_current_buffer(app_runner.app, 'input_buffer')

    def test_return_focus_on_input_buffer(self, app_runner):
        with app_runner.run_app_in_thread():
            app_runner.feed_input(Keys.F5, Keys.F2)
            self.assert_current_buffer(app_runner.app, 'output_buffer')
            app_runner.feed_input(Keys.F5)
            self.assert_current_buffer(app_runner.app, 'input_buffer')



# Copyright 2021 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import botocore.session
import pytest

from awscli.testutils import FileCreator, random_chars
from tests import S3Utils


@pytest.fixture(scope='package')
def region():
    return 'us-west-2'


@pytest.fixture(scope='package')
def session():
    return botocore.session.Session()


@pytest.fixture(scope='package')
def s3_utils(session, region):
    return S3Utils(session, region)


@pytest.fixture(scope='package')
def shared_bucket(s3_utils):
    bucket_name = s3_utils.create_bucket()
    yield bucket_name
    s3_utils.delete_bucket(bucket_name)


# Bucket for cross-S3 copies
@pytest.fixture(scope='package')
def shared_copy_bucket(s3_utils):
    bucket_name = s3_utils.create_bucket()
    yield bucket_name
    s3_utils.delete_bucket(bucket_name)


@pytest.fixture(scope='package')
def shared_dir_bucket(s3_utils):
    bucket_name = s3_utils.create_dir_bucket()
    yield bucket_name
    s3_utils.delete_bucket(bucket_name)


@pytest.fixture(scope='package')
def shared_copy_dir_bucket(s3_utils):
    bucket_name = s3_utils.create_dir_bucket()
    yield bucket_name
    s3_utils.delete_bucket(bucket_name)


# Bucket for cross-region, cross-S3 copies
@pytest.fixture(scope='package')
def shared_cross_region_bucket(s3_utils):
    bucket_name = s3_utils.create_bucket(region='eu-central-1')
    yield bucket_name
    s3_utils.delete_bucket(bucket_name)


@pytest.fixture(scope='package')
def shared_non_dns_compatible_bucket(s3_utils):
    bucket_name = _generate_non_dns_compatible_bucket_name()
    s3_utils.create_bucket(name=bucket_name)
    yield bucket_name
    s3_utils.delete_bucket(bucket_name)


@pytest.fixture(scope='package')
def shared_non_dns_compatible_us_east_1_bucket(s3_utils):
    bucket_name = _generate_non_dns_compatible_bucket_name()
    s3_utils.create_bucket(name=bucket_name, region='us-east-1')
    yield bucket_name
    s3_utils.delete_bucket(bucket_name)


@pytest.fixture
def clean_shared_buckets(
    s3_utils,
    shared_bucket,
    shared_copy_bucket,
    shared_cross_region_bucket,
    shared_non_dns_compatible_bucket,
    shared_non_dns_compatible_us_east_1_bucket,
    shared_dir_bucket,
    shared_copy_dir_bucket,
):
    s3_utils.remove_all_objects(shared_bucket)
    s3_utils.remove_all_objects(shared_copy_bucket)
    s3_utils.remove_all_objects(shared_cross_region_bucket)
    s3_utils.remove_all_objects(shared_non_dns_compatible_bucket)
    s3_utils.remove_all_objects(shared_non_dns_compatible_us_east_1_bucket)
    s3_utils.remove_all_objects(shared_dir_bucket)
    s3_utils.remove_all_objects(shared_copy_dir_bucket)


@pytest.fixture
def files():
    files = FileCreator()
    yield files
    files.remove_all()


def _generate_non_dns_compatible_bucket_name():
    name_comp = []
    # This creates a non DNS compatible bucket name by making two random
    # sequences of characters and joining them with a period and
    # adding a .com at the end.
    for i in range(2):
        name_comp.append(random_chars(10))
    return '.'.join(name_comp + ['com'])



# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from awscli.testutils import BaseAWSCommandParamsTest, unittest


class TestGetApplicationRevisionLocationArguments(BaseAWSCommandParamsTest):
    prefix = 'deploy get-application-revision --application-name foo '

    def test_s3_location(self):
        cmd = self.prefix + '--s3-location bucket=b,key=k,bundleType=zip'
        result = {
            'applicationName': 'foo',
            'revision': {
                'revisionType': 'S3',
                's3Location': {'bucket': 'b', 'key': 'k', 'bundleType': 'zip'},
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_s3_location_with_etag(self):
        cmd = self.prefix + (
            '--s3-location bucket=b,key=k,' 'bundleType=zip,eTag=1234'
        )
        result = {
            'applicationName': 'foo',
            'revision': {
                'revisionType': 'S3',
                's3Location': {
                    'bucket': 'b',
                    'key': 'k',
                    'bundleType': 'zip',
                    'eTag': '1234',
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_s3_location_with_version(self):
        cmd = self.prefix + (
            '--s3-location bucket=b,key=k,' 'bundleType=zip,version=abcd'
        )
        result = {
            'applicationName': 'foo',
            'revision': {
                'revisionType': 'S3',
                's3Location': {
                    'bucket': 'b',
                    'key': 'k',
                    'bundleType': 'zip',
                    'version': 'abcd',
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_s3_location_with_etag_and_version(self):
        cmd = self.prefix + (
            '--s3-location bucket=b,key=k,'
            'bundleType=zip,eTag=1234,version=abcd'
        )
        result = {
            'applicationName': 'foo',
            'revision': {
                'revisionType': 'S3',
                's3Location': {
                    'bucket': 'b',
                    'key': 'k',
                    'bundleType': 'zip',
                    'eTag': '1234',
                    'version': 'abcd',
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_s3_location_json(self):
        cmd = self.prefix + (
            '--s3-location {"bucket":"b","key":"k",'
            '"bundleType":"zip","eTag":"1234","version":"abcd"}'
        )
        result = {
            'applicationName': 'foo',
            'revision': {
                'revisionType': 'S3',
                's3Location': {
                    'bucket': 'b',
                    'key': 'k',
                    'bundleType': 'zip',
                    'eTag': '1234',
                    'version': 'abcd',
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_s3_location_missing_bucket(self):
        cmd = self.prefix + (
            '--s3-location key=k,' 'bundleType=zip,eTag=1234,version=abcd'
        )
        self.run_cmd(cmd, 252)

    def test_s3_location_missing_key(self):
        cmd = self.prefix + (
            '--s3-location bucket=b,' 'bundleType=zip,eTag=1234,version=abcd'
        )
        self.run_cmd(cmd, 252)

    def test_github_location_with_etag(self):
        cmd = self.prefix + (
            '--github-location repository=foo/bar,' 'commitId=1234'
        )
        result = {
            'applicationName': 'foo',
            'revision': {
                'revisionType': 'GitHub',
                'gitHubLocation': {
                    'repository': 'foo/bar',
                    'commitId': '1234',
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_github_location_json(self):
        cmd = self.prefix + (
            '--github-location {"repository":"foo/bar",' '"commitId":"1234"}'
        )
        result = {
            'applicationName': 'foo',
            'revision': {
                'revisionType': 'GitHub',
                'gitHubLocation': {
                    'repository': 'foo/bar',
                    'commitId': '1234',
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_github_location_missing_repository(self):
        cmd = self.prefix + ('--github-location ' 'commitId=1234')
        self.run_cmd(cmd, 252)

    def test_github_location_missing_commit_id(self):
        cmd = self.prefix + '--github-location repository=foo/bar'
        self.run_cmd(cmd, 252)


class TestRegisterApplicationRevisionLocationArguments(
    BaseAWSCommandParamsTest
):
    prefix = 'deploy register-application-revision --application-name foo '

    def test_s3_location(self):
        cmd = self.prefix + '--s3-location bucket=b,key=k,bundleType=zip'
        result = {
            'applicationName': 'foo',
            'revision': {
                'revisionType': 'S3',
                's3Location': {'bucket': 'b', 'key': 'k', 'bundleType': 'zip'},
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_s3_location_with_etag(self):
        cmd = self.prefix + (
            '--s3-location bucket=b,key=k,' 'bundleType=zip,eTag=1234'
        )
        result = {
            'applicationName': 'foo',
            'revision': {
                'revisionType': 'S3',
                's3Location': {
                    'bucket': 'b',
                    'key': 'k',
                    'bundleType': 'zip',
                    'eTag': '1234',
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_s3_location_with_version(self):
        cmd = self.prefix + (
            '--s3-location bucket=b,key=k,' 'bundleType=zip,version=abcd'
        )
        result = {
            'applicationName': 'foo',
            'revision': {
                'revisionType': 'S3',
                's3Location': {
                    'bucket': 'b',
                    'key': 'k',
                    'bundleType': 'zip',
                    'version': 'abcd',
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_s3_location_with_etag_and_version(self):
        cmd = self.prefix + (
            '--s3-location bucket=b,key=k,'
            'bundleType=zip,eTag=1234,version=abcd'
        )
        result = {
            'applicationName': 'foo',
            'revision': {
                'revisionType': 'S3',
                's3Location': {
                    'bucket': 'b',
                    'key': 'k',
                    'bundleType': 'zip',
                    'eTag': '1234',
                    'version': 'abcd',
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_s3_location_json(self):
        cmd = self.prefix + (
            '--s3-location {"bucket":"b","key":"k",'
            '"bundleType":"zip","eTag":"1234","version":"abcd"}'
        )
        result = {
            'applicationName': 'foo',
            'revision': {
                'revisionType': 'S3',
                's3Location': {
                    'bucket': 'b',
                    'key': 'k',
                    'bundleType': 'zip',
                    'eTag': '1234',
                    'version': 'abcd',
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_s3_location_missing_bucket(self):
        cmd = self.prefix + (
            '--s3-location key=k,' 'bundleType=zip,eTag=1234,version=abcd'
        )
        self.run_cmd(cmd, 252)

    def test_s3_location_missing_key(self):
        cmd = self.prefix + (
            '--s3-location bucket=b,' 'bundleType=zip,eTag=1234,version=abcd'
        )
        self.run_cmd(cmd, 252)

    def test_github_location_with_etag(self):
        cmd = self.prefix + (
            '--github-location repository=foo/bar,' 'commitId=1234'
        )
        result = {
            'applicationName': 'foo',
            'revision': {
                'revisionType': 'GitHub',
                'gitHubLocation': {
                    'repository': 'foo/bar',
                    'commitId': '1234',
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_github_location_json(self):
        cmd = self.prefix + (
            '--github-location {"repository":"foo/bar",' '"commitId":"1234"}'
        )
        result = {
            'applicationName': 'foo',
            'revision': {
                'revisionType': 'GitHub',
                'gitHubLocation': {
                    'repository': 'foo/bar',
                    'commitId': '1234',
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_github_location_missing_repository(self):
        cmd = self.prefix + ('--github-location ' 'commitId=1234')
        self.run_cmd(cmd, 252)

    def test_github_location_missing_commit_id(self):
        cmd = self.prefix + '--github-location repository=foo/bar'
        self.run_cmd(cmd, 252)


class TestCreateDeploymentLocationArguments(BaseAWSCommandParamsTest):
    prefix = (
        'deploy create-deployment --application-name foo '
        '--deployment-group bar '
    )

    def test_s3_location(self):
        cmd = self.prefix + '--s3-location bucket=b,key=k,bundleType=zip'
        result = {
            'applicationName': 'foo',
            'deploymentGroupName': 'bar',
            'revision': {
                'revisionType': 'S3',
                's3Location': {'bucket': 'b', 'key': 'k', 'bundleType': 'zip'},
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_s3_location_with_etag(self):
        cmd = self.prefix + (
            '--s3-location bucket=b,key=k,' 'bundleType=zip,eTag=1234'
        )
        result = {
            'applicationName': 'foo',
            'deploymentGroupName': 'bar',
            'revision': {
                'revisionType': 'S3',
                's3Location': {
                    'bucket': 'b',
                    'key': 'k',
                    'bundleType': 'zip',
                    'eTag': '1234',
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_s3_location_with_version(self):
        cmd = self.prefix + (
            '--s3-location bucket=b,key=k,' 'bundleType=zip,version=abcd'
        )
        result = {
            'applicationName': 'foo',
            'deploymentGroupName': 'bar',
            'revision': {
                'revisionType': 'S3',
                's3Location': {
                    'bucket': 'b',
                    'key': 'k',
                    'bundleType': 'zip',
                    'version': 'abcd',
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_s3_location_with_etag_and_version(self):
        cmd = self.prefix + (
            '--s3-location bucket=b,key=k,'
            'bundleType=zip,eTag=1234,version=abcd'
        )
        result = {
            'applicationName': 'foo',
            'deploymentGroupName': 'bar',
            'revision': {
                'revisionType': 'S3',
                's3Location': {
                    'bucket': 'b',
                    'key': 'k',
                    'bundleType': 'zip',
                    'eTag': '1234',
                    'version': 'abcd',
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_s3_location_json(self):
        cmd = self.prefix + (
            '--s3-location {"bucket":"b","key":"k",'
            '"bundleType":"zip","eTag":"1234","version":"abcd"}'
        )
        result = {
            'applicationName': 'foo',
            'deploymentGroupName': 'bar',
            'revision': {
                'revisionType': 'S3',
                's3Location': {
                    'bucket': 'b',
                    'key': 'k',
                    'bundleType': 'zip',
                    'eTag': '1234',
                    'version': 'abcd',
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_s3_location_missing_bucket(self):
        cmd = self.prefix + (
            '--s3-location key=k,' 'bundleType=zip,eTag=1234,version=abcd'
        )
        self.run_cmd(cmd, 252)

    def test_s3_location_missing_key(self):
        cmd = self.prefix + (
            '--s3-location bucket=b,' 'bundleType=zip,eTag=1234,version=abcd'
        )
        self.run_cmd(cmd, 252)

    def test_github_location_with_etag(self):
        cmd = self.prefix + (
            '--github-location repository=foo/bar,' 'commitId=1234'
        )
        result = {
            'applicationName': 'foo',
            'deploymentGroupName': 'bar',
            'revision': {
                'revisionType': 'GitHub',
                'gitHubLocation': {
                    'repository': 'foo/bar',
                    'commitId': '1234',
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_github_location_json(self):
        cmd = self.prefix + (
            '--github-location {"repository":"foo/bar",' '"commitId":"1234"}'
        )
        result = {
            'applicationName': 'foo',
            'deploymentGroupName': 'bar',
            'revision': {
                'revisionType': 'GitHub',
                'gitHubLocation': {
                    'repository': 'foo/bar',
                    'commitId': '1234',
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_github_location_missing_repository(self):
        cmd = self.prefix + ('--github-location ' 'commitId=1234')
        self.run_cmd(cmd, 252)

    def test_github_location_missing_commit_id(self):
        cmd = self.prefix + '--github-location repository=foo/bar'
        self.run_cmd(cmd, 252)


if __name__ == "__main__":
    unittest.main()



# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.clidriver import AWSCLIEntryPoint
from awscli.compat import ensure_text_type
from awscli.testutils import create_clidriver
from tests.functional.history import BaseHistoryCommandParamsTest


class TestListCommand(BaseHistoryCommandParamsTest):
    def test_show_nothing_when_no_history_and_call_made(self):
        self.environ['AWS_CONFIG_FILE'] = ''
        self.driver = create_clidriver()
        self.entry_point = AWSCLIEntryPoint(self.driver)
        self.parsed_responses = [
            {
                "Regions": [
                    {
                        "Endpoint": "ec2.ap-south-1.amazonaws.com",
                        "RegionName": "ap-south-1",
                    },
                ]
            }
        ]
        self.run_cmd('ec2 describe-regions', expected_rc=0)
        stdout, _, _ = self.run_cmd('history show', expected_rc=0)
        # The history show should not display anything as no history should
        # have been collected
        self.assertEqual('', stdout)

    def test_show_nothing_when_no_history(self):
        out, err, rc = self.run_cmd('history list', expected_rc=255)
        error_message = (
            'No commands were found in your history. Make sure you have '
            'enabled history mode by adding "cli_history = enabled" '
            'to the config file.'
        )
        self.assertEqual('', ensure_text_type(out))
        self.assertEqual('\naws: [ERROR]: %s\n' % error_message, ensure_text_type(err))

    def test_show_one_call_present(self):
        self.parsed_responses = [
            {
                "Regions": [
                    {
                        "Endpoint": "ec2.ap-south-1.amazonaws.com",
                        "RegionName": "ap-south-1",
                    },
                ]
            }
        ]
        _, _, rc = self.run_cmd('ec2 describe-regions', expected_rc=0)
        self.history_recorder.record('CLI_RC', rc, 'CLI')
        stdout, _, _ = self.run_cmd('history list', expected_rc=0)
        self.assertIn('ec2 describe-regions', stdout)

    def test_multiple_calls_present(self):
        self.parsed_responses = [
            {
                "Regions": [
                    {
                        "Endpoint": "ec2.ap-south-1.amazonaws.com",
                        "RegionName": "ap-south-1",
                    },
                ]
            },
            {
                "UserId": "foo",
                "Account": "bar",
                "Arn": "arn:aws:iam::1234567:user/baz",
            },
        ]
        _, _, rc = self.run_cmd('ec2 describe-regions', expected_rc=0)
        self.history_recorder.record('CLI_RC', rc, 'CLI')
        _, _, rc = self.run_cmd('sts get-caller-identity', expected_rc=0)
        self.history_recorder.record('CLI_RC', rc, 'CLI')
        stdout, _, _ = self.run_cmd('history list', expected_rc=0)
        self.assertIn('ec2 describe-regions', stdout)
        self.assertIn('sts get-caller-identity', stdout)



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0
import pytest

from awscli.customizations.s3.filegenerator import FileStat
from awscli.customizations.s3.syncstrategy.caseconflict import (
    CaseConflictException,
    CaseConflictSync,
)
from awscli.testutils import skip_if_case_sensitive


@pytest.fixture
def lower_compare_key():
    return 'a.txt'


@pytest.fixture
def lower_temp_filepath(lower_compare_key, tmp_path):
    p = tmp_path / lower_compare_key
    return p.resolve()


@pytest.fixture
def lower_file_stat(lower_compare_key, lower_temp_filepath):
    return FileStat(
        src=f'bucket/{lower_compare_key}',
        dest=lower_temp_filepath,
        compare_key=lower_compare_key,
    )


@pytest.fixture
def upper_compare_key():
    return 'A.txt'


@pytest.fixture
def upper_temp_filepath(upper_compare_key, tmp_path):
    p = tmp_path / upper_compare_key
    p.write_text('foobar')
    return p.resolve()


@pytest.fixture
def upper_file_stat(upper_compare_key, upper_temp_filepath):
    return FileStat(
        src=f'bucket/{upper_compare_key}',
        dest=upper_temp_filepath,
        compare_key=upper_compare_key,
    )


@pytest.fixture
def upper_key(upper_file_stat):
    return upper_file_stat.compare_key.lower()


@pytest.fixture
def no_conflict_file_stat(tmp_path):
    return FileStat(
        src='bucket/foo.txt',
        # Note that this file was never written to.
        dest=f'{tmp_path}/foo.txt',
        compare_key='foo.txt',
    )


class TestCaseConflictSync:
    def test_error_with_no_conflict_syncs(
        self, no_conflict_file_stat, lower_file_stat, capsys
    ):
        case_conflict = CaseConflictSync(on_case_conflict='error')
        should_sync = case_conflict.determine_should_sync(
            no_conflict_file_stat, lower_file_stat
        )
        captured = capsys.readouterr()
        assert should_sync is True
        assert not captured.err

    @skip_if_case_sensitive()
    def test_error_with_existing_file(self, lower_file_stat, upper_file_stat):
        case_conflict_sync = CaseConflictSync(on_case_conflict='error')
        with pytest.raises(CaseConflictException) as exc:
            case_conflict_sync.determine_should_sync(
                lower_file_stat,
                upper_file_stat,
            )
        assert 'Failed to download bucket/a.txt' in str(exc.value)

    def test_error_with_case_conflicts_in_s3(
        self, lower_file_stat, upper_file_stat, upper_key
    ):
        case_conflict_sync = CaseConflictSync(
            on_case_conflict='error', submitted={upper_key}
        )
        with pytest.raises(CaseConflictException) as exc:
            case_conflict_sync.determine_should_sync(
                lower_file_stat,
                upper_file_stat,
            )
        assert 'Failed to download bucket/a.txt' in str(exc.value)

    def test_warn_with_no_conflict_syncs(
        self, no_conflict_file_stat, lower_file_stat, capsys
    ):
        case_conflict = CaseConflictSync(on_case_conflict='warn')
        should_sync = case_conflict.determine_should_sync(
            no_conflict_file_stat, lower_file_stat
        )
        captured = capsys.readouterr()
        assert should_sync is True
        assert not captured.err

    @skip_if_case_sensitive()
    def test_warn_with_existing_file(
        self, lower_file_stat, upper_file_stat, capsys
    ):
        case_conflict_sync = CaseConflictSync(on_case_conflict='warn')
        should_sync = case_conflict_sync.determine_should_sync(
            lower_file_stat, upper_file_stat
        )
        captured = capsys.readouterr()
        assert should_sync is True
        assert 'warning: ' in captured.err

    def test_warn_with_case_conflicts_in_s3(
        self, lower_file_stat, upper_file_stat, upper_key, capsys
    ):
        case_conflict_sync = CaseConflictSync(
            on_case_conflict='warn', submitted={upper_key}
        )
        should_sync = case_conflict_sync.determine_should_sync(
            lower_file_stat,
            upper_file_stat,
        )
        captured = capsys.readouterr()
        assert should_sync is True
        assert 'warning: ' in captured.err

    def test_skip_with_no_conflict_syncs(
        self, no_conflict_file_stat, lower_file_stat, capsys
    ):
        case_conflict = CaseConflictSync(on_case_conflict='skip')
        should_sync = case_conflict.determine_should_sync(
            no_conflict_file_stat, lower_file_stat
        )
        captured = capsys.readouterr()
        assert should_sync is True
        assert not captured.err

    @skip_if_case_sensitive()
    def test_skip_with_existing_file(
        self, lower_file_stat, upper_file_stat, capsys
    ):
        case_conflict_sync = CaseConflictSync(on_case_conflict='skip')
        should_sync = case_conflict_sync.determine_should_sync(
            lower_file_stat, upper_file_stat
        )
        captured = capsys.readouterr()
        assert should_sync is False
        assert 'warning: Skipping' in captured.err

    def test_skip_with_case_conflicts_in_s3(
        self, lower_file_stat, upper_file_stat, upper_key, capsys
    ):
        case_conflict_sync = CaseConflictSync(
            on_case_conflict='skip', submitted={upper_key}
        )
        should_sync = case_conflict_sync.determine_should_sync(
            lower_file_stat,
            upper_file_stat,
        )
        captured = capsys.readouterr()
        assert should_sync is False
        assert 'warning: Skipping' in captured.err



#!/usr/bin/env python
# Copyright 2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import re

from awscli.testutils import BaseAWSCommandParamsTest


class TestWebsiteCommand(BaseAWSCommandParamsTest):
    prefix = 's3 website '

    def test_index_document(self):
        cmdline = self.prefix + 's3://mybucket --index-document index.html'
        result = {
            'WebsiteConfiguration': {
                'IndexDocument': {'Suffix': 'index.html'}
            },
            'Bucket': 'mybucket',
        }

        self.assert_params_for_cmd(cmdline, result)

    def test_error_document(self):
        cmdline = self.prefix + 's3://mybucket --error-document mykey'
        result = {
            'WebsiteConfiguration': {'ErrorDocument': {'Key': 'mykey'}},
            'Bucket': 'mybucket',
        }
        self.assert_params_for_cmd(cmdline, result)


if __name__ == "__main__":
    unittest.main()



# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from botocore.compat import parse_qs, urlparse

from awscli.testutils import BaseAWSCommandParamsTest, FileCreator, mock


class TestSign(BaseAWSCommandParamsTest):
    # A private key only for testing purpose.
    private_key = (
        '-----BEGIN RSA PRIVATE KEY-----\n'
        'MIIEowIBAAKCAQEAu6o2+Jc8UINw2P/w2l7A1xXu3emQEZQ9diA3bmog8r9Dg+65\n'
        'fZgAqmuNWPqBivv7j3DGnLUdt8uCIr7PYUbK7wDa6n7U3ryOWtO2ZTc3StiJVcqT\n'
        'sokZ0qxGFtDRafjBuydXtcxh52vVTcHqH33nubyyZIzuhTwfmrIOnUXnLwbMrBBP\n'
        'bg/8mlgQooyo1XbrN1eO4XMs+UgQ9Mqc7KRJRinUJ+KYuCnM8f/nN4RjYdjTcghk\n'
        'xCPEHCeSt2luywWyYmfguWCBS2Mu1q0250wKyNazlgiiTJtAuuSeweb4NKPOJL9X\n'
        'hR6Ce6UuU4WYlli8gvQh3FAV3N3C1Rxo20k28QIDAQABAoIBAQCUEkP5dWrzpCJg\n'
        'NeHWizjg/L9SfT1dgXfVQqo6BqckoeElsjDNdifgT6hhcpbQEO52SWeMsiNWp85w\n'
        'l9mNSYxJdIVGzPgtHt27sJyT1DNebOg/tu0+y4qCfcd3rR/u24YQo4RDP5ZoQN82\n'
        '0TBn1LIIDWk8iS6SFdRh/OgnE8bLhNbK9IfZQFEEJrFkArrn/le/ro2mfJkC/imo\n'
        'QvqKmM0dGBXt5SCDSbUQAzKtEcR/4gf/qSjFe2YAwAvSA05WXMH6szdtx6/H/VbK\n'
        'Uck/WwTHvGObQDFEWmICxPK9AWT0qaFNjlUsi3bjQRdIlYYrXe+6nVMB/Jp1awq7\n'
        'tGBqIcWBAoGBAPtXCNuoQhKXqkjJgteQpB+wFav12XRZgpOciYdeviJrgWydpOOu\n'
        'O9wkiRUctUijRJbUuWCJF7SgYGoT2xTTp/COiOReqs7qXLMuuXCZcPKkMRJj5wmo\n'
        'Uc2AwUV/o3+PNz1NFK+2RgciXplac7qugIyuxIvBKuVFTBlCg0+if/0pAoGBAL8k\n'
        '845wKqOeiawwle/o9lKLGPy1T11GrE6l1A5jRuE1WTVM77jRrb0Hmo0mdfHaf5A0\n'
        'EjXGIX/fjcmQzBrEd78eCUsvI2Bgn6xXwhd4TTyWHGZfoQjFqAGkixuLN1oo2h1g\n'
        'bRreFKfAubFP8MC93z23vnH6tdY2VIA4h5ehUFyJAoGAJqxJrKLDJ+E2TmTTQR/8\n'
        'YPPTIdZ+UyzCrrvTXYTydJFeJLxM9suEYmcswJbePgMBNsQckgIGJ8DVlPzhJN88\n'
        'ZANKhPkcByKAiQGTfwPdITiqZE4C6rV/gMNi+bKeEa6TrVcC69Z8B/T94VLNo9fd\n'
        '58esbmSWmRiEkQ5u7f3u+6ECgYA8+6ANCLJB43nPCu07TpsP+LrvHTWF799XdEa0\n'
        'lG3vuiKNA8/TqmoAziU79VJZ6Dkcm9BXga/8aSmGboD/5UDDI+UZLJ/fxtQKmzEc\n'
        'ZdBWjRnge5AYCV+xrnqHPiJZzIDSMIp+sO3sG2vjKzsHc0x/F1lWagOLpWfORLrV\n'
        '4KyP6QKBgAafeSrfK3LM7idiCBuxckLCgFoHa7uXLUNJRS5iIU+bbZLPj2ozu/tk\n'
        'U0jp7sNk1CyMWI36lR3sujkSyH3lPIXVgrXMuGY3PJRGntN8WlWEsw4VUMGRj3h4\n'
        '5rB+y/UOS+nlEwQ6eOS09GByJDEXOXpcwjFcTr/f7V8mi0jH+gY/\n'
        '-----END RSA PRIVATE KEY-----\n'
    )
    prefix = 'cloudfront sign --key-pair-id my_id --url http://example.com/hi '

    def setUp(self):
        files = FileCreator()
        self.private_key_file = files.create_file('foo.pem', self.private_key)
        self.addCleanup(files.remove_all)
        super().setUp()

    def assertDesiredUrl(self, url, base, params):
        self.assertEqual(len(url.splitlines()), 1, "Expects only 1 line")
        self.assertTrue(url.startswith(base), "URL mismatch")
        url = url.strip()  # Otherwise the last param contains a trailing CRLF
        self.assertEqual(parse_qs(urlparse(url).query), params)

    def test_canned_policy(self):
        cmdline = (
            self.prefix
            + '--private-key file://'
            + self.private_key_file
            + ' --date-less-than 2016-1-1'
        )
        expected_signature = (
            "UiEmtMsInU-gXoa1O7-bTRJmZ~ocphB0ONMxyEHs2r8Y9dwzeB~DkbgzPMX3jbdb"
            "wIwVX3f4VcY4HBLdPSkbF~D6KbUlxPw1ju8mlXeu2C436XxZdrJrrJaiEDaTpKsl"
            "Xpn9ngaCzVfCVPfkC3a0NBWBySi5ezCG2yzb0c-djNgI1wkogwtmtZuOxAoKF1sR"
            "TyFX9ZitUiUIl~65nkJ94s~GGwxzTf1kMi7Wdm~9rFrJpx0O7nJEBy5O578s2UHr"
            "ejtwyedUR5BqXTkgu~A51NcjAN9LErATV7SVBYicoZ76AOfB-TKay7g6-MWCK6-T"
            "-4Q5x6XH4yzII3JpbCmVwA__"
        )
        expected_params = {
            'Key-Pair-Id': ['my_id'],
            'Expires': ['1451606400'],
            'Signature': [expected_signature],
        }
        self.assertDesiredUrl(
            self.run_cmd(cmdline)[0], 'http://example.com/hi', expected_params
        )

    def test_custom_policy(self):
        cmdline = (
            self.prefix
            + '--private-key file://'
            + self.private_key_file
            + ' --date-less-than 2016-1-1 --ip-address 12.34.56.78'
        )
        expected_signature = (
            "Vw-WG18WJJXim7YSGWS-zW~XmFB9MjCDOvgC~2Gz-1wiMQzCrXzYYbSE7-aF6JGO"
            "Ob5ewArpMqmu2g5mohnqgieZX1NY6IOteDoXYgqaNj1DafHWQD6UJ3IKVfkxISU9"
            "OmFPoG7H~VSPWEzOxdjOqdIPvAU2pW2mJ5oWu2aL62s0VVtLGCAm-DahiSQisl0J"
            "bzpPyG1pofvPbT75qc71r9uiqSAbPjUF5nmLCZazVnFjDkj3zIgMRYa5aV54VDa6"
            "-wEizzmjQ3-m6UMoYgcGHQXEjoFIWTfpZvbZBYkmK9lk3d16cgvaHafTJ-CPegn1"
            "bKxfgNEjSAoPWS0OvBkRmg__"
        )
        expected_params = {
            'Key-Pair-Id': ['my_id'],
            'Policy': [mock.ANY],
            'Signature': [expected_signature],
        }
        self.assertDesiredUrl(
            self.run_cmd(cmdline)[0], 'http://example.com/hi', expected_params
        )


class TestSignPKCS8(BaseAWSCommandParamsTest):
    # A private key only for testing purpose.
    private_key = (
        '-----BEGIN PRIVATE KEY-----\n'
        'MIIEvgIBADANBgkqhkiG9w0BAQEFAASCBKgwggSkAgEAAoIBAQDiVR5JIogE3iKq\n'
        'buYalyKO3vmRnOxf7OU6/8WPma8wpWltb4d67HRBxeUvNugGq0uwinoPDfwF74zG\n'
        'hOKeGrDPLVAbekPzYv1SnB/ppy+nvojDew72xgW56ii9X+Jk83f0TGNTmC7sBvcc\n'
        'kqz3T/aX23NU0faCW5bl6fiW+HVUHZe/aE4nHqhorHiDXlvTV6wpjEWS6Xyf7ll+\n'
        'Jvf4eXg7GqTGTGKsB0jE/xPKdVbnQD67fkJOdaAKTQKanY1UF2SS5Nx6NcBxbcCR\n'
        'Va4myn1JOeQDyHcIXb4NmBx3m21eJSotrJYmD9LTs16mB4wi21lvimALwKxZHjvV\n'
        'p58xKyyJAgMBAAECggEAFtKPdb96KMd/hmEdaeQAk5iPYOwKd9fK+6qL8OGF5Wlg\n'
        'mqzq4+3RAUrjw+GM/xMp1Dj6euclmTGhJ+mBcoDtgE6o68Rl8rZyJfDhVO3LY+ZW\n'
        'IyQXC7JHJIqkpgfzq8tTNrq3L1hCrwE6zNJLh7qz+nciB5UOfvGeYzu3Gf4e0qbi\n'
        'rlStPa7Gi4Oc0EO/51YRjU3IpXjFRvcsqBtV95XA96hPo2ice0KMcrWPF9Kai8bQ\n'
        '0sE+wv+YbgIsbwmnHntdd7Sfxx2jPjXeEgh/ncoXCMYfQueSAHQ/EQBWkofhUeB5\n'
        'oEuQlS5b3D1t3aSKr2o7vrMtu1UWhabu0u+Db/r6gQKBgQD7DKJk0Ow2JBaoM7vV\n'
        'UucuLWLaY4MG4a1YDlHPl6zmD1OioKrQw2h/m2SalYfxM8BjPbR9eesyDv55HQnR\n'
        'ptC1SBNxH7dCwWqCeD1jNVoJP8VkBDPRiNaLz68wYkrtfiXCa0DYbewbdrEFDaIk\n'
        'IErrRzxSWTSNE8Y1YA3ka6MiaQKBgQDmy7TdLa0tyYwY30DmLmS4WUZglJZKrT/0\n'
        'd9UTz7KJek7P9BNZAe8yotVrxO2di+8W85GAVQBexeISrEW6ZK6GHGz949fJmbvq\n'
        'QOU/6TgE01AL0nUZF2QKbdAleonlR/WB9IpZTQf/ZI1HmUV0QL3nCrs9OoFbzx4E\n'
        'GfjbCmQ1IQKBgFOlZgZJRirT42ivtAnj0XslTCaPuXx1fRg1zTRpyQXuXWN2PPPJ\n'
        '5+t8jwyifeTz5UorqROVp7PKIyefcUIVXrzIAxJSCvGHGEHYZjvD7vfd85rbe5h5\n'
        'C2MSE8D/Pw/aVCJvMe/q0Bxmc5zHahq3V78EwSh+6G+JAyWNl5Nf+b7hAoGBAM1Z\n'
        'PGB7DpYpuLw8j9r+NmGMFUFDk4F4KupSYMTSzPDjYRJIAZr1TKWKGkhcHGtMIXwT\n'
        'VUeQ2dZ5TM/+dcAFav8qdZNk0Q+v+HHSMeeuk0g/1/3c0JF1rW5WDJf8MotNflSV\n'
        'hy8zicUj60xkRFbOb+kNNFGjJ4vPec5+aVxDH6vhAoGBAI3RsJZXYUL9PhakrsVp\n'
        '71N+JbNxvw8L9b2VL6ecLNMtPcG5ddFaMhc+kQZap6vAZXauft1fzvAO3fMKNJXm\n'
        'yvtM2CEYzVd8lFqA8xETa/FgelkFjB5gkiq4EDIuX6mFStkskKUfRHHrb0ATKHSl\n'
        'YvT60qFc4be2Mfyzt+CuGhYi\n'
        '-----END PRIVATE KEY-----\n'
    )
    prefix = 'cloudfront sign --key-pair-id my_id --url http://example.com/hi '

    def setUp(self):
        files = FileCreator()
        self.private_key_file = files.create_file('foo.pem', self.private_key)
        self.addCleanup(files.remove_all)
        super().setUp()

    def assertDesiredUrl(self, url, base, params):
        self.assertEqual(len(url.splitlines()), 1, "Expects only 1 line")
        self.assertTrue(url.startswith(base), "URL mismatch")
        url = url.strip()  # Otherwise the last param contains a trailing CRLF
        self.assertEqual(parse_qs(urlparse(url).query), params)

    def test_canned_policy(self):
        cmdline = (
            self.prefix
            + '--private-key file://'
            + self.private_key_file
            + ' --date-less-than 2016-1-1'
        )
        expected_signature = (
            "cIOcUXezjLknta66EiRX7rk3viXv20F01OwZa1X2QWxhnWnBVno~mg0Gcyfzvfgo"
            "-oXCvZC3bdsfTJXiBcnC1XyxCxBa03bouAae4A0ajP4ey~TKKwPHikOmu2Rc1NEu"
            "-c6wr8DbMZrm~1WIWG4kFG1jhSRoEk2W82NkGEh4xEPq3gaNjQPfF7zIAwcZUUkg"
            "GkIbT-cQ5UZ6rTqTiFGdXD2z8kjulgmtu8Quo6hplch~9ltmKTOt9blswd6hMfCM"
            "NJ~tUj77j8fz968adb9w43jBtl~~5seb8ys01cg5IGWV44LKMWaLmEgzWQAjg-Jg"
            "9wx-HYwuqH4Klds03WZzRQ__"
        )
        expected_params = {
            'Key-Pair-Id': ['my_id'],
            'Expires': ['1451606400'],
            'Signature': [expected_signature],
        }
        self.assertDesiredUrl(
            self.run_cmd(cmdline)[0], 'http://example.com/hi', expected_params
        )

    def test_custom_policy(self):
        cmdline = (
            self.prefix
            + '--private-key file://'
            + self.private_key_file
            + ' --date-less-than 2016-1-1 --ip-address 12.34.56.78'
        )
        expected_signature = (
            "beEwE8ZmSX71e79a5dxupiE0zHxahe1IFzuTExKxV0InQnKFlT0wj0tardAlGKFL"
            "LdX9HMGiVjIjvMBdUZQJ-9mMXBtFsQ5nLDEoRH29H8AATzaf4Nx4n29XtVp-jPVF"
            "GFtmdaGJedjJRMV-IzBQcJ19VPl3R8t3Fp~8eP9-P8KpvkJXH2UvJ2H8nMBt2Ogv"
            "brCT2hl~91UtEOgmxeA6twWNpziH0uEdpDOHgnYer5ScdFoo02rPjRXIqPuQcjwP"
            "T2wu~A5T~zomcghjMcIdLeJeS9nscTkjON69xBB-t4lclK3mfzsXTumcx-FzLgOB"
            "bP2Z1d~ZU6X0rkeL~w1BlQ__"
        )
        expected_params = {
            'Key-Pair-Id': ['my_id'],
            'Policy': [mock.ANY],
            'Signature': [expected_signature],
        }
        self.assertDesiredUrl(
            self.run_cmd(cmdline)[0], 'http://example.com/hi', expected_params
        )



# Copyright 2021 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import importlib
import os

import pytest

import awscli
from awscli import TopLevelImportAliasFinder

AWSCLI_ROOT_DIR = os.path.dirname(awscli.__file__)


@pytest.fixture()
def fake_builtin_path_finder():
    class PathFinder(type):
        __module__ = '_frozen_importlib_external'

    return PathFinder


@pytest.fixture()
def fake_pyinstaller_finder():
    class PyiFrozenImporter:
        __module__ = 'pyimod02_importers'

    return PyiFrozenImporter()


class RecordingMetaPathFinder(importlib.abc.MetaPathFinder):
    def __init__(self):
        self.find_spec_calls = []

    def find_spec(self, fullname, path, target=None):
        self.find_spec_calls.append((fullname, path, target))
        return None


@pytest.mark.parametrize(
    'fullname,provided_path,expected_path',
    [
        ('botocore', None, [AWSCLI_ROOT_DIR]),
        ('s3transfer', None, [AWSCLI_ROOT_DIR]),
        ('awscli', None, None),
        ('awscli.utils', [AWSCLI_ROOT_DIR], [AWSCLI_ROOT_DIR]),
        ('logging', None, None),
    ],
)
def test_find_spec(fullname, provided_path, expected_path):
    underlying_finder = RecordingMetaPathFinder()
    alias_finder = TopLevelImportAliasFinder(underlying_finder)
    alias_finder.find_spec(fullname, provided_path, target=None)
    assert underlying_finder.find_spec_calls == [
        (fullname, expected_path, None)
    ]


def test_add_alias_finder_wraps_builtin_finder(fake_builtin_path_finder):
    sys_meta = [fake_builtin_path_finder]
    TopLevelImportAliasFinder.add_alias_finder(sys_meta)
    _assert_classes(
        sys_meta, [TopLevelImportAliasFinder, type(fake_builtin_path_finder)]
    )


def test_add_alias_finder_wraps_pyinstaller_finder(fake_pyinstaller_finder):
    sys_meta = [fake_pyinstaller_finder]
    TopLevelImportAliasFinder.add_alias_finder(sys_meta)
    _assert_classes(
        sys_meta, [TopLevelImportAliasFinder, type(fake_pyinstaller_finder)]
    )


def test_add_alias_finder_does_not_wrap_other_finders():
    do_not_wrap_this_finder = RecordingMetaPathFinder()
    sys_meta = [do_not_wrap_this_finder]
    TopLevelImportAliasFinder.add_alias_finder(sys_meta)
    assert sys_meta == [do_not_wrap_this_finder]


def test_add_alias_finder_wraps_only_first_matching_finder(
    fake_pyinstaller_finder,
):
    sys_meta = [
        RecordingMetaPathFinder(),
        fake_pyinstaller_finder,
        fake_pyinstaller_finder,
    ]
    TopLevelImportAliasFinder.add_alias_finder(sys_meta)
    _assert_classes(
        sys_meta,
        [
            RecordingMetaPathFinder,
            TopLevelImportAliasFinder,
            fake_pyinstaller_finder.__class__,
            fake_pyinstaller_finder.__class__,
        ],
    )


def _assert_classes(sys_meta, expected_classes):
    assert [finder.__class__ for finder in sys_meta] == expected_classes
